# -*-
#
# @author: jaumebonet
# @email:  jaume.bonet@gmail.com
# @url:    jaumebonet.github.io
#
# @date:   2015-05-11 13:28:48
# @lab:    LPDI/EPFL
#
# @last modified by:   jaumebonet
# @last modified time: 2015-05-11 13:40:29
#
# -*-

'''
Given a PIR alignment and a minimal set of parameters generates models
through homology modeling with MODELLER.

In this instance, it allows to "force" secondary structure definitions and/or
distance constraints.

[!] The script does not contain any input check.
[!] The restrained_model function redirects STDOUT and STDERR to file logs.
    The redirection is terminated before the function ends.

'''
import sys
import os
import warnings
from argparse import ArgumentParser

from modpyHelper import basic_parser
from modpyHelper import identify_pir

from modeller import *
from modeller.automodel import *


def set_options(*args, **kargs):
    '''
    Set the specific options for this script

    @return: parsed ArgumentParser
    '''
    parser = ArgumentParser(parents = [basic_parser()],
                            conflict_handler='resolve')

    # Setting forced alpha-helix --> each entry is a tuple; repetitions store several tuples
    parser.add_argument("--ahelix", dest="ahelix", action="append", type=int, nargs=2,
                         help="Regions forced to be alpha helix", metavar="A_ini A_end")
    # Setting forced beta-strand --> each entry is a tuple; repetitions store several tuples
    parser.add_argument("--bstrand", dest="bstrand", action="append", type=int, nargs=2,
                         help="Regions forced to be abeta strand", metavar="S_ini S_end")
    # Fixing distances between CA --> each entry is a tuple; repetitions store several tuples
    parser.add_argument("--distance", dest="distance", action="append", nargs=3,
                        type=float, metavar="CA1 CA2 Dist",
                         help="Fixed distance between two CA, specify as: CA1 CA2 Distance")

    parser.add_argument("--models", dest="numMod",     action="store",
                        type=int,   metavar="MOD_NUM", default=5,
                        help="Number of models to be done (min: 1) [Default: 5]")

    parser.add_argument("--optimize",  dest="optimize", action="store_true",
                        default=False, help="Activate optimization")
    return parser.parse_args()


def restrained_model(alignment, instances, helices=None, betas=None, distances=None,
                     output=None, optimize=False, verbose=Falese):

    contents = identify_pir(alignment)

    # We redirect STDOUT and STDERR to output files
    if output is None: output = contents['seq']
    warnings.warn('STDOUT & STDERR will be redirected to log files.')
    sys.stdout = open(output + '.log', 'w')
    sys.stderr = open(output + '.err', 'w')

    if verbose: log.verbose()  # Commands MODELLER to display all log output

    env = environ()  # Initializes the 'environment' for this modeling run

    env.io.hetatm = True  # Allow the presence of hetatoms

    class MyModel(automodel):
        def special_restraints(self, aln):
            rsr = self.restraints
            at = self.atoms
            # Setting forced alpha helices
            try:
                for helix in helices:
                    rsr.add(secondary_structure.alpha(self.residue_range(str(helix[0])+':',
                                                                         str(helix[1])+':')))
            except TypeError: pass

            # Setting forced beta strands
            try:
                for strand in betas:
                    rsr.add(secondary_structure.strand(self.residue_range(str(strand[0])+':',
                                                                          str(strand[1])+':')))
            except TypeError: pass

            # Setting distance restraints
            try:
                for restraint in distances:
                    rsr.add(forms.gaussian(group=physical.xy_distance,
                                           feature=features.distance(at['CA:' + str(restraint[0])],
                                                                     at['CA:' + str(restraint[1])]),
                                           mean=float(restraint[2]),
                                           stdev=0.1)
                            )
            except TypeError: pass

    a = automodel(env,  # Loading the environment
                  alnfile=alignment,         # Assigning the PIR alignment
                  knowns=contents['str'],    # Listing the known structures
                  sequence=contents['seq'],  # Identify the Query Sequence
                  assess_methods=(assess.DOPE,
                                  assess.GA341,
                                  assess.DOPEHR))  # Energy evaluation methods

    # Setting Starting and Ending Model number
    a.starting_model, a.ending_model = 1, instances

    if optimize:
        # Very thorough VTFM optimization:
        a.library_schedule   = autosched.slow
        a.max_var_iterations = 300

        # Thorough MD optimization:
        a.md_level = refine.slow

        # Repeat the whole cycle 2 times and do not stop unless obj.func. > 1E6
        a.repeat_optimization = 2
        a.max_molpdf          = 1e6

    a.initial_malign3d = True

    a.make()  # Create the Model

    sys.stdout = sys.__stdout__  # restore stdout back to normal
    sys.stderr = sys.__stderr__  # restore stdout back to normal
    warnings.warn('STDOUT & STDERR have been restored.')

    # We delete the error file if no error has occurred
    ERRsize = os.path.getsize(output + '.err')
    if int(ERRsize) is 0:
        os.remove(output + '.err')

if __name__ == '__main__':
    options = set_options()
    restrained_model(options.alignment, options.numMod, options.out,
                     options.optimize, options.verbose)



import sys
import argparse
import os
import re
from Bio import SeqIO as SeqIE
from Bio import ExPASy
from Bio.Seq import Seq
from Bio import SeqRecord
from Bio.Alphabet import IUPAC
import ConfigParser
import itertools
import shutil
import subprocess
import time
import random


# Add '.' to sys.path
src_path = os.path.abspath(os.path.dirname(__file__))
sys.path.append(src_path)

# Read configuration file
config = ConfigParser.ConfigParser()
config_file = os.path.join(src_path, 'config.ini')
config.read(config_file)

# Add SBI library path to sys.path
src_path = os.path.join(config.get('Paths','modppi_path'),config.get('Paths','sbi_library_path'))
sys.path.append(src_path)

# Add functions path to sys.path
src_path = os.path.join(config.get('Paths','modppi_path'),config.get('Paths','functions_path'))
sys.path.append(src_path)

# Import functions
from SeqIO import *
from functions import *
from SBI.external.blast import blast_parser
from SBI.structure.contacts import Complex
from SBI.sequence import Sequence
from SBI.structure import PDB

class ModelException(Exception):
    pass


def main():

    start_time = time.time()
    options    = parse_user_arguments()
    verbose    = options.show
    clean      = options.clean
    dummy_dir  = options.dummy_dir
    output_dir = options.outdir
    renumerate = options.renumerate

    #Parse input data and create databases if necessary
    queriesA, queriesB, all_edges, user_defined_edges = parse_inputs(options)
    queriesA_original=queriesA
    queriesB_original=queriesB

    #Blast and model all requested interactions
    start_time_init = time.time()
    for user_edge in user_defined_edges:
        current_query_A={}
        current_query_B={}
        if not queriesA.has_key(user_edge[0]) or not queriesB.has_key(user_edge[1]): continue
        current_query_A.setdefault(user_edge[0],queriesA[user_edge[0]])
        current_query_B.setdefault(user_edge[1],queriesB[user_edge[1]])
        hit_items_A, hit_items_B = do_blast(options, current_query_A, current_query_B)
        sections_modeled = {}
        remaining_sections_A = {}
        remaining_sections_B = {}
        for hit_A in hit_items_A:
            for hit_B in hit_items_B:
                # If the interaction is not listed in any of the PPI files, skip to the next interaction
                if user_edge != (hit_A[0], hit_B[0]) and user_edge != (hit_B[0], hit_A[0]): continue
                #if verbose: sys.stdout.write('\t\t\t ... Check edge %s in hits %s : %s \n'%(user_edge,hit_A[1],hit_B[1]))
                if (hit_A[1], hit_B[1]) not in all_edges and (hit_B[1], hit_A[1]) not in all_edges: continue
                #if (hit_A[0], hit_B[0]) not in user_defined_edges and (hit_B[0], hit_A[0]) not in user_defined_edges: continue
                start_time_model = time.time()
                try:
                    sections_modeled, remaining_sections_A, remaining_sections_B = modelling(queriesA_original,queriesB_original, queriesA, queriesB, hit_A, hit_B, sections_modeled, remaining_sections_A, remaining_sections_B, options)
                except ModelException:
                    continue
                if verbose: sys.stdout.write('\t\t\tTotal time on model %s (template %s): %f seconds\n' %(user_edge[0]+"::"+user_edge[1],hit_A[1][:-2],(time.time() - start_time_model)))
        if not sections_modeled:
            has_modeled = False
        else:
            has_modeled = True
        sys.stdout.write('\t\tAccumulated time on main modelling of %s: %d seconds\n' %(user_edge[0]+"::"+user_edge[1],(time.time() - start_time_init)))

        start_time_remain = time.time()
        # Do all the models of this interaction before proceeding to the next
        while remaining_sections_A or remaining_sections_B:
            queriesA_remaining = remaining_sections_A
            queriesB_remaining = remaining_sections_B
            hit_items_A, hit_items_B = do_blast(options, queriesA_remaining, queriesB_remaining)
            sections_modeled = {}
            remaining_sections_A = {}
            remaining_sections_B = {}
            for hit_A in hit_items_A:
                for hit_B in hit_items_B:
                    # If the interaction is not listed in the PPI file, skip to the next interaction
                    if user_edge != (hit_A[0], hit_B[0]) and user_edge != (hit_B[0], hit_A[0]): continue
                    if (hit_A[1], hit_B[1]) not in all_edges and (hit_B[1], hit_A[1]) not in all_edges: continue
                    start_time_model = time.time()
                    try:
                        sections_modeled, remaining_sections_A, remaining_sections_B = modelling(queriesA_original,queriesB_original, queriesA_remaining, queriesB_remaining, hit_A, hit_B, sections_modeled, remaining_sections_A, remaining_sections_B, options)
                    except ModelException:
                        continue
                    if verbose: sys.stdout.write('\t\t\tTotal time on model %s (template %s): %f seconds\n' %(user_edge[0]+"::"+user_edge[1],hit_A[1][:-2],(time.time() - start_time_model)))
        sys.stdout.write('\t\tAccumulated time on secondary regions modelling of %s: %d seconds\n' %(user_edge[0]+"::"+user_edge[1],(time.time() - start_time_remain)))
        with open(os.path.join(options.outdir, 'interactions_done.list'), 'a+') as interactions_done_file:
            if not re.search('\\b%s\t%s\\b' %(user_edge[0], user_edge[1]), interactions_done_file.read()):
                if has_modeled:
                    interactions_done_file.write('%s\t%s\tDONE\t%s\n' %(user_edge[0], user_edge[1],os.path.abspath(os.path.join(options.outdir,user_edge[0]+"::"+user_edge[1],user_edge[0]+"::"+user_edge[1]+".list"))))
                else:
                    interactions_done_file.write('%s\t%s\tFAILED\n' %(user_edge[0], user_edge[1]))
            else:
                if has_modeled:
                    interactions_done_file.write('%s\t%s\tFIXED\t%s\n' %(user_edge[0], user_edge[1],os.path.abspath(os.path.join(options.outdir,user_edge[0]+"::"+user_edge[1],user_edge[0]+"::"+user_edge[1]+".list"))))

    if not verbose: shutil.rmtree(dummy_dir)
    if verbose and clean: 
         sys.stdout.write('Cleaning folder %s\n'%dummy_dir)
         shutil.rmtree(dummy_dir)
    sys.stdout.write('Total elapsed time: %d seconds\n' %(time.time() - start_time))


def parse_user_arguments(*args, **kwds):
    parser = argparse.ArgumentParser(
        description = 'Automatic modelling of protein-protein interactions',
        epilog      = '@Oliva\'s lab 2016')
    parser.add_argument('-seq', '--query', dest = 'query_file', action = 'store',
                        help = 'Input file with a list of FASTA sequences of the proteins to test')
    parser.add_argument('-ppi', '--interactions', dest = 'interactions_file', action = 'store',
                        help = 'Input file with a list of the interactions to test')
    parser.add_argument('-o', '--output_directory', dest = 'outdir', action = 'store', default = 'ModPPI_models',
                        help = 'Output directory (default is ModPPI_models)')
    parser.add_argument('-n', '--number_of_models', dest = 'nmodels', action = 'store', default = 1, type=int,
                        help = 'Number of models for each template (default is 1)')
    parser.add_argument('-d', '--dummy_dir', dest = 'dummy_dir', action = 'store', default = '/tmp/modppi_dummy',
                        help = 'Specifies the location of the dummy folder (default is /tmp/modppi_dummy)')
    parser.add_argument('-opt', '--optimize', dest = 'optimize', action = 'store_true',
                        help = 'Flag to allow model optimization (default is False)')
    parser.add_argument('-3did', '--use_domain_interactions', dest = 'did', action = 'store_true',
                        help = 'Flag to include domain-domain interactions from 3DiD (default is False)')
    parser.add_argument('-init_pdb', '--initialize_PDB_dataset', dest = 'init_pdb',action = 'store_true',
                        help = 'Flag to force initialization of dataset from PDB (default is False)')
    parser.add_argument('-init_3did', '--initialize_3did_dataset', dest = 'init_3did',action = 'store_true',
                        help = 'Flag to force initialization of dataset from 3DiD (default is False)')
    parser.add_argument('-init_blast', '--initialize_blast_dataset', dest = 'init_blast',action = 'store_true',
                        help = 'Flag to force formatting the sequence dataset for BLAST (default is False)')
    parser.add_argument('-skip', '--skip_protein_check', dest = 'skip_check',action = 'store_true',
                        help = 'Flag to skip the check of the sequence dataset and fasten up the modeling (default is False)')
    parser.add_argument('-hydro','--hydrogens', dest = 'hbplus', action = 'store_true',
                        help = 'Flag to include hydrogens (default is False)')
    parser.add_argument('-force','--force', dest = 'force', action = 'store_true',
                        help = 'Flag to force the modelling even if the model already exist (default is False)')
    parser.add_argument('-clean','--force_clean_dummy', dest = 'clean', action = 'store_true',
                        help = 'Flag to force removing the dummy files (default is False)')
    parser.add_argument('-r','--renumerate', dest = 'renumerate' , action = 'store_true',
                        help = 'Flag to renumber the sequences as in the original FastA (default is False)')
    parser.add_argument('-v', '--verbose', dest = 'show', action = 'store_true',
                        help = 'Flag for verbose mode (default is False)')
    options = parser.parse_args()
    return options



def parse_inputs(options):

    # Initialize
    verbose = options.show
    output_dir = options.outdir
    dummy_dir = options.dummy_dir
    init_pdb=options.init_pdb
    init_3did=options.init_3did
    init_blast=options.init_blast
    skip_check=options.skip_check
    make_database=init_pdb or init_3did or init_blast
    src_path = config.get('Paths','modppi_path')
    blast_path = config.get('Paths', 'blast_path')
    blast_dir  = os.path.join(src_path, config.get('Paths', 'blast_dir'))
    queriesA = {}
    queriesB = {}

    # Create output subdirectories
    if not os.path.exists(output_dir) and fileExist(options.interactions_file) and fileExist(options.query_file):
        os.makedirs(output_dir)
    if not os.path.exists(blast_dir):
        os.makedirs(blast_dir)
    if not os.path.exists(dummy_dir):
        os.makedirs(dummy_dir)


    # skipping the check of sequences is inconsistent with initialization
    if init_pdb or init_3did or init_blast: skip_check=False

    if not skip_check:
      # Store and ckeck the dataset sequences
      proteins = {}
      protein_names = set()
      fasta_list_file = os.path.join(src_path,config.get('Files', 'fasta_list_file'))
      if fileExist(fasta_list_file):
        if verbose: sys.stdout.write('\t\t-- Reading protein sequences...\n')
        for protein in FASTA_iterator(fasta_list_file):
            name = protein.get_identifier()
            seq = protein.get_sequence()
            if len(seq) > 0:
                try:
                    proteins.setdefault(name, ProteinSequence(name, seq))
                    protein_names.add(name)
                except IncorrectSequenceLetter as e:
                    sys.stderr.write('WARNING: %s\n' %e)
                    sys.stdout.write('\t\t-- Skip input sequence: %s Sequence: %s\n' %(name, seq))
      else:
        sys.stderr.write('EXIT: Missing NR sequence list file %s\n' %(fasta_list_file))
        exit(0)

      nr90_names = set()
      nr90_list_file = os.path.join(src_path,config.get('Files', 'nr90_list_file'))
      if fileExist(nr90_list_file):
        if verbose: sys.stdout.write('\t\t-- Reading non-redundant sequences...\n')
        for protein in FASTA_iterator(nr90_list_file):
            name = protein.get_identifier()
            seq = protein.get_sequence()
            if len(seq) > 0:
                try:
                    proteins.setdefault(name, ProteinSequence(name, seq))
                    nr90_names.add(name)
                except IncorrectSequenceLetter as e:
                    if verbose: sys.stderr.write('WARNING: %s\n' %e)
                    if verbose: sys.stdout.write('\t\t-- Skip input sequence: %s Sequence: %s\n' %(name, seq))
      else:
        sys.stderr.write('EXIT: Missing NR sequence list file %s\n' %(nr90_list_file))
        exit(0)


    # Create PPI file; skip if it already exists unless forcing initialization
    ppi_files = os.path.join(src_path,config.get('Files', 'ppi_files'))
    ppi_files_edges=ppi_files+".ppi"
    ppi_files_dat  =ppi_files+".dat"
    if not fileExist(ppi_files_edges) or init_pdb:
        if verbose: sys.stdout.write('\t\t-- Writing PPI file...\n')
        pdb_list_file = config.get('Files', 'pdb_list_file')
        functions_path=config.get('Paths', 'functions_path')
        python_path = config.get('Paths', 'python_path')
        data_path = os.path.join(src_path,config.get('Paths', 'data_path'))
        PPI_distance_threshold = config.get('Parameters', 'PPI_distance_threshold')
        PPI_threshold_type = config.get('Parameters', 'PPI_threshold_type')
        pdb2pin=os.path.join(functions_path,'PDB2PIN.py')
        # Create PPI file using 'PDB2PIN.py' script
        if verbose:
         process = subprocess.Popen([os.path.join(python_path, 'python'), pdb2pin, '-i', pdb_list_file, '-o', ppi_files, '-v' , '--PPI_distance', PPI_distance_threshold, '--PPI_type', PPI_threshold_type], stderr = subprocess.STDOUT)
         process.wait()
        else:
         process = subprocess.Popen([os.path.join(python_path, 'python'), pdb2pin, '-i', pdb_list_file, '-o', ppi_files, '--PPI_distance', PPI_distance_threshold, '--PPI_type', PPI_threshold_type], stderr = subprocess.STDOUT)
         process.wait()


    # if we have skipped the check of sequences we only need to store the edges without checking the existence of a proper protein sequence
    if not skip_check:
      # 'pdb_edges' must be a set so the script can run faster
      pdb_edges = set((x, y) for (x, y, alt_a, alt_b) in PPI_iterator(ppi_files_edges) if x in protein_names and y in protein_names)
      all_edges = pdb_edges
      nodes = get_nodes(pdb_edges)
    else:
      # User needs to be warned
      sys.stderr.write("WARNING: the background set of PPIs is untested \n")
      # 'pdb_edges' must be a set so the script can run faster
      pdb_edges = set((x, y) for (x, y, alt_a, alt_b) in PPI_iterator(ppi_files_edges))
      all_edges = pdb_edges
      nodes = get_nodes(pdb_edges)

     

    #Inlcude domain interactions upon requesting option flag
    if options.did:
      # Create DDI file; skip if it already exists
      ddi_files = os.path.join(src_path,config.get('Files', 'ddi_files'))
      ddi_files_edges=ddi_files+".ppi"
      ddi_files_dat  =ddi_files+".dat"
      if not fileExist(ddi_files_edges) or init_3did:
        if verbose: sys.stdout.write('\t\t-- Writing DDI file...\n')
        did_list_file = os.path.join(src_path,config.get('Files', '3did_list_file'))
        python_path = config.get('Paths', 'python_path')
        functions_path=os.path.join(src_path,config.get('Paths', 'functions_path'))
        data_path = os.path.join(src_path,config.get('Paths', 'data_path'))
        PPI_distance_threshold = config.get('Parameters', 'PPI_distance_threshold')
        PPI_threshold_type = config.get('Parameters', 'PPI_threshold_type')
        pdb2pin=os.path.join(functions_path,'PDB2PIN.py')
        # Create PPI file using 'PDB2PIN.py' script
        if verbose:
          process = subprocess.Popen([os.path.join(python_path, 'python'), pdb2pin, '-i', did_list_file, '-o', ddi_files, '-v' , '--PPI_distance', PPI_distance_threshold, '--PPI_type', PPI_threshold_type], stderr = subprocess.STDOUT)
          process.wait()
        else:
          process = subprocess.Popen([os.path.join(python_path, 'python'), pdb2pin, '-i', did_list_file, '-o', ddi_files, '--PPI_distance', PPI_distance_threshold, '--PPI_type', PPI_threshold_type], stderr = subprocess.STDOUT)
          process.wait()

      # if we have skipped the check of sequences we only need to store the edges without checking the existence of a proper protein sequence
      if not skip_check:
        # 'ddi_edges' is also a set and we update the set of nodes with domains
        ddi_edges = set((x, y) for (x, y, alt_a, alt_b) in PPI_iterator(ddi_files_edges) if x in protein_names and y in protein_names)
        all_edges.update(ddi_edges)
        nodes.update(get_nodes(ddi_edges))
      else:
        # User needs to be warned
        sys.stderr.write("WARNING: the background set of DDIs is untested \n")
        # 'ddi_edges' is also a set and we update the set of nodes with domains
        ddi_edges = set((x, y) for (x, y, alt_a, alt_b) in PPI_iterator(ddi_files_edges) )
        all_edges.update(ddi_edges)
        nodes.update(get_nodes(ddi_edges))

    # obtain edges as dictionary
    edges={}
    for x,y in all_edges:
       edges.setdefault(x,set()).add(y)
       edges.setdefault(y,set()).add(x)

    # Format BLAST database of PIN; skip if it already exists
    blast_dir  = os.path.join(src_path, config.get('Paths', 'blast_dir'))
    database  = config.get('Files', 'database_file')
    database_file = os.path.join(blast_dir, database)
    included={}
    if not fileExist(database_file) or make_database:
        if fileExist(nr90_list_file+".clstr"):
           if verbose: sys.stdout.write('\t\t-- Reading clusters of redundancy...\n')
           cluster_id=parse_clusters(nr90_list_file+".clstr")
        else: cluster_id={}
        if verbose: sys.stdout.write('\t\t-- Building database of sequences...\n')
        if nodes is not None:
            out = open(database_file , 'w')
            for name in nodes:
              if name in nr90_names:
                printfasta(out, name, proteins[name].get_sequence())
                included.setdefault(name,True) 
            add=set()
            for name in included.keys():
                #Select the clusters of interactors of the selected proteins (set cset)
                cset=set(cluster_id[x] for x in edges[name] if included.has_key(x) and cluster_id.has_key(x))
                #Find proteins which homologs are not yet included or proteins that were not clustered (set iset)
                iset=set()
                for x in edges[name]:
                   if included.has_key(x):continue
                   if cluster_id.has_key(x): 
                      if cluster_id[x] not in cset: 
                         iset.add(x)
                         #Add the cluster of x in cset
                         cset.add(cluster_id[x])
                   else: iset.add(x)
                add.update(iset)
            while (len(add)>0):
              if verbose: sys.stdout.write('\t\t\t Missing %d nodes. Add and check for new\n'%(len(add)))
              #Include in the list the missed proteins
              for name in add:
                 printfasta(out, name, proteins[name].get_sequence())
                 included.setdefault(name,True)
              add_new=set()
              for name in add:
                cset=set(cluster_id[x] for x in edges[name] if included.has_key(x) and cluster_id.has_key(x))
                iset=set()
                for x in edges[name]:
                   if included.has_key(x):continue
                   if cluster_id.has_key(x): 
                      if cluster_id[x] not in cset:
                         iset.add(x)
                         #Add the cluster of x in cset
                         cset.add(cluster_id[x])
                   else: iset.add(x)
                add_new.update(iset)
              add=add_new
            out.close()
            # Format database
            if verbose: sys.stdout.write("\t\t-- Formatting and indexing the database...\n\t\t   %s\n"%(blast_path + 'formatdb'+ ' -i '+ database_file))
            process = subprocess.Popen([blast_path + 'formatdb', '-i', database_file], stderr = subprocess.STDOUT)
            process.wait()
            # Remove files created upon execution
            remove_files(['formatdb.log'])
    else:
         sys.stderr.write("WARNING: the database of sequences already exists. Be aware you are using the last created file for BLAST\n")

    # Read the interactions file and create groups A and B
    if verbose: sys.stdout.write('\t\t-- Reading input PPI file...\n')
    # Set with all user-given interactions
    user_interactions = set()
    if fileExist(options.interactions_file):
        protein_pairs=open(options.interactions_file, 'r')
        group_A = []
        group_B = []
        for pair in protein_pairs:
                group_A.append(pair.split('\t')[0])
                group_B.append(pair.split('\t')[1].strip())
                given_interaction = (pair.split('\t')[0], pair.split('\t')[1].strip())
                user_interactions.add(given_interaction)
        protein_pairs.close()
    else:
        sys.stderr.write('EXIT: Missing input PPI file %s\n' %(options.interactions_file))
        exit(0)


    # Read and store sequences of query sequences file in the corresponding groups
    if verbose: sys.stdout.write('\t\t-- Reading query sequences file...\n')
    if fileExist(options.query_file):
        for protein in FASTA_iterator(options.query_file):
            name = protein.get_identifier()
            # If the header has a UniProt pattern, 'name' will be its accession number
            if re.search('[a-z][|A-Z0-9][|A-Z0-9]', name):
               if len(name.split('|'))>1:
                name = name.split('|')[1]
            seq = protein.get_sequence()
            if len(seq) > 0:
                try:
                    if name in group_A:
                        queriesA.setdefault(name, ProteinSequence(name, seq))
                    if name in group_B:
                        queriesB.setdefault(name, ProteinSequence(name, seq))
                except IncorrectSequenceLetter as e:
                    sys.stderr.write('WARNING: %s\n' %e)
                    if verbose: sys.stdout.write('\t\t -- Skip input sequence: %s Sequence: %s\n' %(name, seq))
    else:
        sys.stderr.write('EXIT: Missing query sequences file %s\n' %(options.query_file))
        exit(0)


    return queriesA, queriesB, all_edges, user_interactions

def parse_clusters(clstr):
    cl=open(clstr,"r")
    cluster_id={}
    for line in cl:
      if line.startswith(">Cluster"): cluster_name=line.strip().split()[1]
      m = re.search("(\w+), >(\w+)...", line)
      if m:
         cluster_id.setdefault(m.group(2),cluster_name)
    cl.close()
    return cluster_id
         

def do_blast(options, queriesA, queriesB):
    # Initialize
    verbose = options.show
    dummy_dir = options.dummy_dir
    blast_results_dir = os.path.join(dummy_dir, 'blast')
    make_subdirs(dummy_dir, subdirs = ['blast'])
    src_path = config.get('Paths','modppi_path')
    blast_path = os.path.join(src_path, config.get('Paths', 'blast_path'))
    blast_dir  = os.path.join(src_path, config.get('Paths', 'blast_dir'))
    database_file = os.path.join(blast_dir, './database.fasta')
    if verbose: sys.stdout.write('\t\t-- BLAST of Query A and Query B files...\n')
   

    # QUERIES A
    hit_items_A = []
    # Store blast hits of query A sequences
    for name in queriesA:
        # Create FASTA file for name in BLAST directory
        fasta_file = os.path.join(blast_results_dir, name + '.fa')
        seq_queryA = queriesA[name].get_sequence()
        if not os.path.exists(fasta_file):
            fd = open(fasta_file, 'w')
            printfasta(fd, name, seq_queryA)
            fd.close()
        # Define blast output name
        blast_file = os.path.join(blast_results_dir, name + '.blast.out')
        skip = False
        # Execute blastpgp unless it already exists an output
        if not os.path.exists(blast_file):
            try:
                if verbose: sys.stdout.write('\t\t\t-- Blast of %s...\n' %(fasta_file))
                if seq_queryA:
                    #if verbose: sys.stdout.write('\t\t\t   %s -d %s -i %s -m 7 -o %s\n'%(os.path.join(blast_path, 'blastpgp'),database_file,fasta_file,blast_file))
                    process = subprocess.Popen([os.path.join(blast_path, 'blastpgp'), '-d', database_file, '-i', fasta_file, '-m 7', '-o', blast_file], stderr = subprocess.STDOUT)
                    process.wait()
            except:
                sys.stderr.write('WARNING: "blastpgp on %s" execution failed!\n' %(name))
                skip = True
            if not os.path.exists(blast_file):
                sys.stderr.write('WARNING: No blast output on %s could be found!\n' %(name))
                skip = True
        # If blast output exists for <name> FASTA
        if not skip:
            done = set()
            # Get blast object
            if verbose: sys.stdout.write('\t\t\t-- Parsing BLAST %s...\n' %(blast_file))
            try:
              blast_obj = blast_parser.parse_blast(query_sequence = None, blast_output_file = blast_file, selfHit = False, hitIDformat = 'all')
            except Exception as e:
              sys.stderr.write('ERROR: BLAST output  %s collapsed; remove "dummy" folder and run again when all queues are done\n'%(blast_file))
            # Filter hits by Rost's sequence identity curve
            n_parameter_twilight_zone = config.get('Parameters', 'n_parameter_twilight_zone')
            # For each hit...
            n = -1
            for hit in blast_obj.get_hits(tz_parameter = n_parameter_twilight_zone, tz_type = 'ID'):
                # Skip if already done hit
                n += 1
                if hit.sequenceID in done: continue
                # Add hit to done
                done.add(hit.sequenceID)
                # Check if hit is equal to query
                if blast_obj.query.split(':')[0] == hit.sequenceID.split('_')[0]: continue
                # Store hit items for using them in the modelling process
                hit_items_A.append((blast_obj.query, hit.sequenceID, hit.query_seq, hit.hit_seq, hit.query_pos, hit.hit_pos))


    # QUERIES B
    hit_items_B = []
    # Store blast hits of query B sequences
    for name in queriesB:
        # Create FASTA file for name in BLAST directory
        fasta_file = os.path.join(blast_results_dir, name + '.fa')
        seq_queryB = queriesB[name].get_sequence()
        if not os.path.exists(fasta_file):
            fd = open(fasta_file, 'w')
            printfasta(fd, name, seq_queryB)
            fd.close()
        # Define blast output name
        blast_file = os.path.join(blast_results_dir, name + '.blast.out')
        skip = False
        # Execute blastpgp unless it already exists an output
        if not os.path.exists(blast_file):
            try:
                if verbose: sys.stdout.write('\t\t\t-- Blast of %s...\n' %(fasta_file))
                if seq_queryB:
                    #if verbose: sys.stdout.write('\t\t\t   %s -d %s -i %s -m 7 -o %s\n'%(os.path.join(blast_path, 'blastpgp'),database_file,fasta_file,blast_file))
                    process = subprocess.Popen([os.path.join(blast_path, 'blastpgp'), '-d', database_file, '-i', fasta_file, '-m 7', '-o', blast_file], stderr = subprocess.STDOUT)
                    process.wait()
            except:
                sys.stderr.write('WARNING: "blastpgp on %s" execution failed!\n' %(name))
                skip = True
            if not os.path.exists(blast_file):
                sys.stderr.write('WARNING: No blast output on %s could be found!\n' %(name))
                skip = True
        # If blast output exists for <name> FASTA
        if not skip:
            done = set()
            # Get blast object
            if verbose: sys.stdout.write('\t\t\t-- Parsing BLAST %s...\n' %(blast_file))
            try:
              blast_obj = blast_parser.parse_blast(query_sequence = None, blast_output_file = blast_file, selfHit = False, hitIDformat = 'all')
            except Exception as e:
              sys.stderr.write('ERROR: BLAST output  %s collapsed; remove "dummy" folder and run again when all queues are done\n'%(blast_file))
            # Filter hits by Rost's sequence identity curve
            n_parameter_twilight_zone = config.get('Parameters', 'n_parameter_twilight_zone')
            # For each hit...
            n = -1
            for hit in blast_obj.get_hits(tz_parameter = n_parameter_twilight_zone, tz_type = 'ID'):
                # Skip if already done hit
                n += 1
                if hit.sequenceID in done: continue
                # Add hit to done
                done.add(hit.sequenceID)
                # Check if hit is equal to query
                if blast_obj.query.split(':')[0] == hit.sequenceID.split('_')[0]: continue
                # Store hit items for using them in the modelling process
                hit_items_B.append((blast_obj.query, hit.sequenceID, hit.query_seq, hit.hit_seq, hit.query_pos, hit.hit_pos))


    return hit_items_A, hit_items_B


def modelling(queriesA_original,queriesB_original,queriesA, queriesB, hit_items_A, hit_items_B, sections_modeled, remaining_sections_A, remaining_sections_B, options):
    # Initialize
    verbose = options.show
    output_dir = options.outdir
    dummy_dir = options.dummy_dir
    hydrogens = options.hbplus
    force_model =options.force
    python_path = config.get('Paths', 'python_path')
    src_path = config.get('Paths','modppi_path')
    modeller_path = os.path.join(config.get('Paths', 'modeller_path'))
    modpy_path = os.path.join(src_path, config.get('Paths', 'functions_path'),"modpy")
    numMod= options.nmodels
    renumerate = options.renumerate


    # Assign the PID to the dummy modeling and avoid overwritting files 
    
    modelling_dummy_name = 'modelling_' + str(os.getpid()) + str(random.randint(0,os.getpid()))
    #modelling_dummy_name = 'modelling_' + str(os.getpid())
    make_subdirs(dummy_dir, subdirs = [modelling_dummy_name])
    modelling_dir = os.path.join(dummy_dir, modelling_dummy_name)


    # Get items from the hits
    query_A_orig = queriesA_original.get(hit_items_A[0])
    query_B_orig = queriesB_original.get(hit_items_B[0])
    query_A = queriesA.get(hit_items_A[0]).get_sequence()
    query_B = queriesB.get(hit_items_B[0]).get_sequence()
    query_name_A = hit_items_A[0]
    query_name_B = hit_items_B[0]
    query_id_A = query_name_A.split(':')[0]
    query_start = hit_items_A[4][0]
    query_end = int(hit_items_A[4][-1]) + int(hit_items_B[4][-1])
    template_name_A_chain = hit_items_A[1]
    template_name_B_chain = hit_items_B[1]
    template_chain_A_chain = template_name_A_chain.split('_')[-1]
    template_chain_B_chain = template_name_B_chain.split('_')[-1]
    template_A_chain_start = hit_items_A[5][0]
    template_B_chain_start = hit_items_B[5][0]
    template_id_A = "_".join(template_name_A_chain.split('_')[:-1])
    template_id_B = "_".join(template_name_B_chain.split('_')[:-1])
    sequences_complex = {}
    sequences_complex.setdefault("A",query_A_orig)
    sequences_complex.setdefault("B",query_B_orig)

    # Get the positions of the current section
    extension_threshold = int(config.get('Parameters', 'extension_threshold'))
    current_A_section = [hit_items_A[4][0], hit_items_A[4][-1]]
    current_B_section = [hit_items_B[4][0], hit_items_B[4][-1]]
    current_sections = [current_A_section, current_B_section]
    current_interaction = '%s::%s' %(query_name_A, query_name_B)

    # Initialize 'sections_modeled' dictionary
    if not sections_modeled.get(current_interaction):
        section_group = sections_modeled.setdefault(current_interaction, [])
        section_group.append(current_sections)

    # Check if the segments of the current interaction belong to a previous group
    for section_pair in sections_modeled.get(current_interaction):
        # The segments must be within a given interval
        if (section_pair[0][0] - extension_threshold <= current_sections[0][0] <= section_pair[0][0] + extension_threshold and
            section_pair[0][1] - extension_threshold <= current_sections[0][1] <= section_pair[0][1] + extension_threshold and
            section_pair[1][0] - extension_threshold <= current_sections[1][0] <= section_pair[1][0] + extension_threshold and
            section_pair[1][1] - extension_threshold <= current_sections[1][1] <= section_pair[1][1] + extension_threshold):
                current_sections = section_pair
                break
    # If the segments are not within the interval, create a new group
    else:
        section_group = sections_modeled.setdefault(current_interaction, [])
        section_group.append(current_sections)

    # Get the sections that have not been used in the alignment
    query_A_fragment_used = hit_items_A[2].replace('-', '')
    query_B_fragment_used = hit_items_B[2].replace('-', '')
    remaining_terminus_A = query_A.split(query_A_fragment_used)
    remaining_terminus_B = query_B.split(query_B_fragment_used)
    Nterminus_name_A = '%s_1-%s' %(query_name_A, hit_items_A[4][0] - 1)
    Cterminus_name_A = '%s_%s-%s' %(query_name_A, hit_items_A[4][-1] + 1, len(query_A))
    Nterminus_name_B = '%s_1-%s' %(query_name_B, hit_items_B[4][0] - 1)
    Cterminus_name_B = '%s_%s-%s' %(query_name_B, hit_items_B[4][-1] + 1, len(query_B))

    # If there are remaining sections, store them in the dictionary
    if hit_items_A[4][0] > 1:
        remaining_sections_A[Nterminus_name_A] = ProteinSequence(Nterminus_name_A, remaining_terminus_A[0])
    if hit_items_A[4][-1] < len(query_A):
        remaining_sections_A[Cterminus_name_A] = ProteinSequence(Cterminus_name_A, remaining_terminus_A[-1])
    if hit_items_B[4][0] > 1:
        remaining_sections_B[Nterminus_name_B] = ProteinSequence(Nterminus_name_B, remaining_terminus_B[0])
    if hit_items_B[4][-1] < len(query_B):
        remaining_sections_B[Cterminus_name_B] = ProteinSequence(Cterminus_name_B, remaining_terminus_B[-1])

    #Create LOG for tests
    if verbose:
     dummy_log_file="%s/%s.log"%(modelling_dir, template_id_A)
     dummy_log=open(dummy_log_file,"a")

    # Create PDB file
    if verbose: sys.stdout.write('\t\t-- Using templates %s and %s...\n' %(template_name_A_chain, template_name_B_chain))
    pdb_name = template_id_A
    dummy_pdb_file = '%s/%s.pdb' %(modelling_dir, pdb_name.replace(":","-"))
    # Initialize PDB object
    pdb_obj = PDB()


    #Check template in PDB files
    src_path = config.get('Paths','modppi_path')
    pdb_path = os.path.join(src_path, config.get('Paths', 'pdb_path'), template_id_A[1:3].lower())
    pdb_file = os.path.join(pdb_path, 'pdb' + template_id_A.lower() + '.ent')
    if not os.path.exists(pdb_file):
      sys.stderr.write('WARNING: PDB file %s was not found, try compressed\n' %(pdb_file))
      pdb_file = os.path.join(pdb_path, 'pdb' + template_id_A.lower() + '.ent.gz')

    #Check now template in 3DiD files
    if not os.path.exists(pdb_file):
      sys.stderr.write('WARNING: PDB file %s was not found, try 3DiD ".brk" suffix\n' %(pdb_file))
      pdb_path = os.path.join(src_path, config.get('Paths', '3did_path'))
      pdb_file = os.path.join(pdb_path, template_id_A.lower() + '.brk')
      if not os.path.exists(pdb_file):
        sys.stderr.write('WARNING: PDB file %s was not found, try 3DiD ".brk" suffix compressed\n' %(pdb_file))
        pdb_file = os.path.join(pdb_path, template_id_A.lower() + '.brk.gz')

    # If the PDB file is not found in the database, skips to the next interaction
    if not os.path.exists(pdb_file):
        sys.stderr.write('WARNING: PDB file %s was not found\n' %(pdb_file))
        raise ModelException
    pdb_chain_obj = PDB(pdb_file)
    pdb_chain_obj.clean()

    # Add only the chains present in the alignment
    pdb_obj.add_chain(pdb_chain_obj.get_chain_by_id(template_chain_A_chain))
    pdb_obj.add_chain(pdb_chain_obj.get_chain_by_id(template_chain_B_chain))

    # Get sequences from PDB, where 'x' are gaps and 'X' are heteroatoms
    pdb_seqA = pdb_obj.chains[0].gapped_protein_sequence.replace('x', '-').replace('X', '.')
    pdb_seqB = pdb_obj.chains[1].gapped_protein_sequence.replace('x', '-').replace('X', '.')

    # Create the dummy PDB file
    pdb_obj.clean()
    pdb_obj.write(output_file = dummy_pdb_file,force=True)

    # Check contacts
    PPI_threshold_type = config.get('Parameters', 'PPI_threshold_type')
    PPI_distance_threshold = float(config.get('Parameters', 'PPI_distance_threshold'))
    protein_complex = Complex(pdb_obj, PPI_type = PPI_threshold_type, PPI_distance = PPI_distance_threshold)
    # If the proteins don't form a complex, avoids modelling
    if len(protein_complex.PPInterfaces[0].contacts) == 0:
        sys.stderr.write('WARNING: No interaction between %s and %s ( for %s %s)\n' %(template_name_A_chain, template_name_B_chain, query_name_A, query_name_B))
        remove_files([dummy_pdb_file])
        raise ModelException
    else:
        if verbose: sys.stdout.write('\t\t\t-- Accepted interaction between %s and %s ( for %s %s)...\n' %(template_name_A_chain, template_name_B_chain, query_name_A, query_name_B))

    # Correct possible discrepancies between the template sequence found in the FASTA file of the nodes in the PIN and the sequence found in the PDB file
    # e.g. The sequence of a protein can have an 'X' in the FASTA file and an 'M' in the newest version of the PDB file
    template_seqA = hit_items_A[3]
    template_seqA_ungapped = re.sub('-', '', template_seqA)
    pdbA_section = pdb_seqA[hit_items_A[5][0]-1:hit_items_A[5][-1]]
    for pair in itertools.izip(template_seqA_ungapped, pdbA_section):
        if pair[0] == 'X' or pair[0] == 'x':
            template_seqA = re.sub('[xX]', pair[1], template_seqA, 1)
    template_seqB = hit_items_B[3]
    template_seqB_ungapped = re.sub('-', '', template_seqB)
    pdbB_section = pdb_seqB[hit_items_B[5][0]-1:hit_items_B[5][-1]]
    for pair in itertools.izip(template_seqB_ungapped, pdbB_section):
        if pair[0] == 'X' or pair[0] == 'x':
            template_seqB = re.sub('[xX]', pair[1], template_seqB, 1)

    if verbose: dummy_log.write("Hits_items_A: %s\n"%([str(x) for x in hit_items_A]))
    if verbose: dummy_log.write("Hits_items_B: %s\n"%([str(x) for x in hit_items_B]))
    if verbose: dummy_log.write("pdbA_section %s\n"%pdbA_section)
    if verbose: dummy_log.write("pdbB_section %s\n"%pdbB_section)
    if verbose: dummy_log.write("length PDB A: %d\n"%len(pdb_seqA))
    if verbose: dummy_log.write("length PDB B: %d\n"%len(pdb_seqB))

    # Add the remaining residues at the beginning or at the end of the template sequences, if needed
    template_seqA = re.sub('[xX]', '-', template_seqA)
    if template_A_chain_start > 1:
        template_A_first_residues = ''.join(pdb_seqA[:hit_items_A[5][0]-1])
        template_seqA = template_A_first_residues + template_seqA
    if hit_items_A[5][-1] < len(pdb_seqA):
        template_seqA += ''.join(pdb_seqA[hit_items_A[5][-1]:])
    template_seqB = re.sub('[xX]', '-', template_seqB)
    if template_B_chain_start > 1:
        template_B_first_residues = ''.join(pdb_seqB[:hit_items_B[5][0]-1])
        template_seqB = template_B_first_residues + template_seqB
    if hit_items_B[5][-1] < len(pdb_seqB):
        template_seqB += ''.join(pdb_seqB[hit_items_B[5][-1]:])


    if verbose: dummy_log.write("FINAL template_seqA %s\n"%template_seqA)
    if verbose: dummy_log.write("FINAL template_seqB %s\n"%template_seqB)

    # Add gaps at the beginning of the query sequences, if needed
    gaps_number_A_chain_beginning = 0
    gaps_number_B_chain_beginning = 0
    if template_A_chain_start > 1:
        gaps_number_A_chain_beginning = int(template_A_chain_start) - 1
    if template_B_chain_start > 1:
        gaps_number_B_chain_beginning = int(template_B_chain_start) - 1
    A_chain_query_seq = ''.join(['-' for i in range(gaps_number_A_chain_beginning)]) + re.sub('[xX]', '-', hit_items_A[2])
    B_chain_query_seq = ''.join(['-' for i in range(gaps_number_B_chain_beginning)]) + re.sub('[xX]', '-', hit_items_B[2])

    # Add gaps at the end of the query sequences, if needed
    for pair in itertools.izip_longest(A_chain_query_seq, template_seqA):
        if pair[0] == None:
            A_chain_query_seq += '-'
    for pair in itertools.izip_longest(B_chain_query_seq, template_seqB):
        if pair[0] == None:
            B_chain_query_seq += '-'


    # Create PIR alignment
    query_whole_seq = A_chain_query_seq + '/' + B_chain_query_seq + '*'
    template_whole_seq = template_seqA + '/' + template_seqB + '*'

    header1 = '>P1;%s\nsequence:%s:%s:.:%s:.:.:.:.:.' %(query_id_A, query_id_A, query_start, query_end)
    header2 = '>P1;%s\nstructureX:%s:1:%s:.:%s:.:.:.:.' %(template_id_A.replace(":","-"), template_id_A.replace(":","-"), template_chain_A_chain, template_chain_B_chain)

    lines = []
    lines.append(header1)
    lines.extend([query_whole_seq[i:i+60] for i in range(0, len(query_whole_seq), 60)])
    lines.append(header2)
    lines.extend([template_whole_seq[i:i+60] for i in range(0, len(template_whole_seq), 60)])

    pir_alignment = '\n'.join(lines)
    pir_file = open('%s/alignment.pir' %(modelling_dir), 'w+')
    for line in lines:
        pir_file.write('%s\n' %(line))
    pir_file.close()


    # Model
    # Create a folder for the models of each type of interaction
    if '-' in query_name_A:
        query_name_A = query_name_A.rsplit('_', 1)[0]
    if '-' in query_name_B:
        query_name_B = query_name_B.rsplit('_', 1)[0]
    interaction_dir = os.path.join(output_dir , '%s::%s' %(query_name_A, query_name_B))
    if not os.path.exists(interaction_dir):
        make_subdirs(output_dir, subdirs = ['./%s::%s' %(query_name_A, query_name_B)])

    # If the models do not yet exist, proceed and add in the list of MODELS
    do_model=False
    model_path = os.path.abspath(interaction_dir)
    for imodel in xrange(1,numMod+1):
     model_name = '%s_%s_%d-%d::%s_%s_%d-%d#%d.pdb' %(template_id_A, template_chain_A_chain, current_sections[0][0], current_sections[0][1], template_id_B, template_chain_B_chain, current_sections[1][0], current_sections[1][1],imodel)
     model_path_model = os.path.join(model_path , model_name)
     #print "CHECK %s %s\n"%(do_model,model_path_model)
     with open(interaction_dir + '/%s.list' %(current_interaction), 'a+') as paths_to_models_file:
        if model_path_model not in paths_to_models_file.read(): paths_to_models_file.write(model_path_model + '\n')
     if not do_model and not fileExist( model_path_model ): do_model=True

    #Complete the set of models
    if do_model or force_model:
        # Keep the current working directory, then change to the modelling folder
        cwd = os.getcwd()
        os.chdir(modelling_dir)
        try:
         if options.optimize:
            process = subprocess.check_output([os.path.join(modeller_path, 'modpy.sh'), os.path.join(python_path, 'python'), os.path.join(modpy_path, 'simpleModel.py'), '--pir=' + './alignment.pir', '--out=%s-%s' %(template_name_A_chain, template_name_B_chain), '--models=%d'%(numMod), '--optimize'], stderr = subprocess.STDOUT)
         else:
            process = subprocess.check_output([os.path.join(modeller_path, 'modpy.sh'), os.path.join(python_path, 'python'), os.path.join(modpy_path, 'simpleModel.py'), '--pir=' + './alignment.pir', '--out=%s-%s' %(template_name_A_chain, template_name_B_chain), '--models=%d'%(numMod)], stderr = subprocess.STDOUT)
        except Exception as e:
         sys.stderr.write("ERROR: %s\n"%(e))
         sys.stderr.write("LOCATION; %s\n"%modelling_dir)
         if verbose: os.system("grep get_ran %s"%(template_name_A_chain+"-"+template_name_B_chain+".log"))
         if verbose: sys.stderr.write("\t\tSkip models with template %s\n"%(model_name))
         os.chdir(cwd)
         raise ModelException
        # Clean and rename all models
        for imodel in xrange(1,numMod+1):
          label_model=99990000+imodel
          input_model = '%s.B%s.pdb' %(query_id_A,str(label_model))
          model_name = '%s_%s_%d-%d::%s_%s_%d-%d#%d.pdb' %(template_id_A, template_chain_A_chain, current_sections[0][0], current_sections[0][1], template_id_B, template_chain_B_chain, current_sections[1][0], current_sections[1][1],imodel)
          model_path_model = os.path.join(model_path , model_name)
          if fileExist(os.path.abspath('%s' %(input_model))):
            # Check contacts
            check_pdb_obj=PDB(os.path.abspath('%s' %(input_model)))
            PPI_threshold_type = config.get('Parameters', 'PPI_threshold_type')
            PPI_distance_threshold = float(config.get('Parameters', 'PPI_distance_threshold'))
            check_protein_complex = Complex(check_pdb_obj, PPI_type = PPI_threshold_type, PPI_distance = PPI_distance_threshold)
            if len(check_protein_complex.PPInterfaces[0].contacts) == 0:
              if verbose: sys.stdout.write("\t\t\t-- Skip model without contacts %s\n"%model_name)
              continue
            else:
              if verbose: sys.stdout.write("\t\t\t-- Accepted model %s\n"%model_name)
            if hydrogens:
              if verbose: sys.stdout.write("\t\t\t-- Adding hydrogens and relaxing the model %s\n"%model_name)
              output_model=model_name
              try:
               add_hydrogens(config,os.path.abspath("./"),input_model, output_model,dummy_dir)
              except ValueError as e:
               sys.stderr.write("WARNING %s\n"%e)
               os.rename(input_model, output_model)
            else:
              output_model=model_name
              os.rename(input_model, output_model)
            if renumerate:
              if verbose: sys.stdout.write("\t\t\t-- Renumerate residues as original sequence\n")
              output_model_renumber=model_name+".re"
              try:
                pdb_renumber=PDB()
                pdb_renumber=renumber_pdb(config,os.path.abspath("./"),output_model,sequences_complex,os.path.abspath("./"))
                pdb_renumber.write(output_model_renumber)
                os.rename(output_model_renumber,output_model)
              except Exception as e:
                sys.stderr.write("WARNING %s\n"%e)
            shutil.copy(output_model, model_path_model)
        os.chdir(cwd)
    try:
     shutil.rmtree(modelling_dir)
    except Exception as e:
     sys.stderr.write("WARNING first attempt to remove folder %s\n"%e)
     try:
       os.system("\\rm -r %s"%(modelling_dir))
     except Exception as ee:
       sys.stderr.write("WARNING last attempt %s\n"%ee)
      
      

    return sections_modeled, remaining_sections_A, remaining_sections_B


  

# MAIN ###########################################################################################################

if __name__ == '__main__':
    main()



from SBI.databases import Enzyme

tables = {'enz'      : 'enzyme',
          'enz_ext'  : 'enzyme_extended',
          'enz_reac' : 'enzyme_reaction',
          'enz_com'  : 'enzyme_reaction_compounds',
          'enz_cop'  : 'enzyme_compound',
          'enz_cof'  : 'enzyme_cofactor',
          'enz_trs'  : 'enzyme_transfered', 
          'unienz'   : 'uniprot2enzyme'}

class Enzyme(Enzyme):
    def __init__(self, ec = None, description = None, inline = None):
        super(Enzyme, self).__init__(ec, description, inline)

    def toSQL(self):
        command = ""
        for cof in self.cofactors:
            command += "INSERT IGNORE INTO {0} SET name ='{1}', type='cofactor';\n".format(tables['enz_cop'], cof)
        for comp in self.compounds:
            command += "INSERT IGNORE INTO {0} SET name ='{1}', type='compound';\n".format(tables['enz_cop'], comp)

        if self.has_direct_parent and not self.has_transfers and not self.is_deleted:
            command += "INSERT INTO {0} VALUES ('{1.ec}','{1.description}',{1.level},'{1.direct_p}');\n".format(tables['enz'], self)
        else:
            command += "INSERT INTO {0} (id, description, level) VALUES ('{1.ec}','{1.description}',{1.level});\n".format(tables['enz'], self)
        
        if not self.has_transfers and not self.is_deleted:
            for parent in self.parents:
                command += "INSERT INTO {0} VALUES ('{1}','{2.ec}');\n".format(tables['enz_ext'], parent, self)

        for cofactor in self.cofactors:
            command += "SET @cof = (SELECT nid FROM {0} WHERE name='{1}' AND type='cofactor');\n".format(tables['enz_cop'], cofactor)
            command += "INSERT INTO {0} VALUES ('{1.ec}', @cof);\n".format(tables['enz_cof'], self, cofactor)

        for reaction in self.reactions:
            if not reaction.repeated:
                command += "INSERT INTO {0} (ec, description) VALUES ('{1.ec}','{2.description}');\n".format(tables['enz_reac'], self, reaction)
            else:
                command += "SET @old_id = (SELECT nid FROM {0} WHERE description='{1.description}' LIMIT 1);\n".format(tables['enz_reac'], reaction)
                command += "INSERT INTO {0} VALUES (@old_id, '{1.ec}','{2.description}');\n".format(tables['enz_reac'], self, reaction)
            for subst in reaction.substrates:
                command += "SET @compnum = (SELECT nid FROM {0} WHERE name='{1[0]}' AND type='compound');\n".format(tables['enz_cop'], subst)
                command += "INSERT INTO {0} VALUES (LAST_INSERT_ID(),@compnum,'substrate',{1[1]});\n".format(tables['enz_com'], subst)
            for prod in reaction.products:
                command += "SET @compnum = (SELECT nid FROM {0} WHERE name='{1[0]}' AND type='compound');\n".format(tables['enz_cop'], prod)
                command += "INSERT INTO {0} VALUES (LAST_INSERT_ID(),@compnum,'product',{1[1]});\n".format(tables['enz_com'], prod)
        for protein in self.uniprots:
            command += "INSERT INTO {0} VALUES('{1}','{2}');\n".format(tables['unienz'], protein, self.ec)
        return command

    def transfered2SQL(self):
        command = ""
        for trans in self.transfers:
            command += "INSERT INTO {0} VALUES ('{1.ec}','{2}');\n".format(tables['enz_trs'], self, trans)
        return command



import os, sys, random, math
import numpy
from BioLib.Tools.BioExceptions import ResidueTriadError

class SplitPotentialsPPI(object):
    '''
    Split Potentials for protein-protein interctions. 
    Applications: Evaluate ppi, rank them (docking) and find the hot spots.
    '''
    def __init__(self, c_type = 'CB', cutoff = 12):
        '''
        Contructor
        '''
        c_type_accepted = ('CB','MIN')
        if c_type.upper() not in c_type_accepted:
            sys.stderr.write('WARNING: Incorrect contact type %s, using CB!' % c_type)
            c_type = 'CB'

        if c_type == 'CB':
            self.sp_file = os.path.abspath(os.path.join(os.path.dirname(__file__), 'ppi3_cbR12.out'))
        else:
            self.sp_file = os.path.abspath(os.path.join(os.path.dirname(__file__), 'ppi3_minR5.out'))
        if not os.path.isfile(self.sp_file):
            sys.stderr.write('ERROR: Can not find the split potential file %s\n\n' % self.sp_file)
            sys.exit()

        self.c_type = c_type.upper()  # Contact type
        self.cutoff = cutoff          # Standard cutoffs are 12 for CB and 5 for MIN

        self.ppR     = {}             # Pair Potentials of Residues
        self.ppE     = {}             # Pair Potentials of Environment
        self.LppRE   = {}             # Local Pair Potentials of Residues and Environment
        self.ppRE    = {}             # Pair Potentials of Residues and Environment
        self.ppD     = {}             # Standard Pair Potentials of Distances

        self.read_sp_file()           # Read the file and load the matrices
        
    ##################
    # PUBLIC METHODS #
    ##################

    def calculate_global_energies(self, interaction, Zscores=False, randoms=100, funnel=False):
        '''
        Compute the split potentials energies for a protein-protein interaction
        if Zscores: Computes also the Zscores
        if funnel: Compute also the external and partial energies (shuffling the interface with non-interacting exposed residues)
        '''
        # Compute global energies
        real_interactions = interaction.get_interacting_residues(c_type=self.c_type, max_distance=self.cutoff+1, uniq=False)
        real_energies = self.calculate_energies(zip(real_interactions[0], real_interactions[1], real_interactions[2]))
        interaction_info = [real_energies]
        if funnel:
            noninterating_exposedR = self.__get_noninteracting_residues(list(set(real_interactions[0])), interaction.get_structure1())
            noninterating_exposedL = self.__get_noninteracting_residues(list(set(real_interactions[1])), interaction.get_structure2())
            if not noninterating_exposedR or not noninterating_exposedL:
                raise RuntimeError('ERROR: Non noninteracting exposed residues (bad ACC assignation?)')
            native_energies_lists = {}
            external_energies_lists = {}
            partial_energies_lists = {}
            native_energies = {}
            external_energies = {}
            partial_energies = {}
            for random in xrange(randoms):
                rand_nonint_R = self.randomize_residue_interface(noninterating_exposedR, real_interactions[0])
                rand_nonint_L = self.randomize_residue_interface(noninterating_exposedL, real_interactions[1])
                rand_int_R = self.randomize_residue_interface(list(set(real_interactions[0])), real_interactions[0])
                rand_int_L = self.randomize_residue_interface(list(set(real_interactions[1])), real_interactions[1])
                rand_native_energies = self.calculate_energies(zip(rand_int_R, rand_int_L, real_interactions[2], real_interactions[0], real_interactions[1]), randomized=True)
                rand_partialR_energies = self.calculate_energies(zip(rand_int_R, rand_nonint_L, real_interactions[2], real_interactions[0], real_interactions[1]), randomized=True)
                rand_partialL_energies = self.calculate_energies(zip(rand_nonint_R, rand_int_L, real_interactions[2], real_interactions[0], real_interactions[1]), randomized=True)
                rand_external_energies = self.calculate_energies(zip(rand_nonint_R, rand_nonint_L, real_interactions[2], real_interactions[0], real_interactions[1]), randomized=True)
                for potential_type in rand_native_energies:
                    native_energies_lists.setdefault(potential_type, []).append(rand_native_energies[potential_type])  
                for potential_type in rand_partialR_energies:
                    partial_energies_lists.setdefault(potential_type, []).append(rand_partialR_energies[potential_type])
                for potential_type in rand_partialL_energies:
                    partial_energies_lists.setdefault(potential_type, []).append(rand_partialL_energies[potential_type]) 
                for potential_type in rand_external_energies:
                    external_energies_lists.setdefault(potential_type, []).append(rand_external_energies[potential_type])
            for potential in native_energies_lists:
                native_energies[potential] = numpy.mean(native_energies_lists[potential])
            for potential in partial_energies_lists:
                partial_energies[potential] = numpy.mean(partial_energies_lists[potential])
            for potential in external_energies_lists:
                external_energies[potential] = numpy.mean(external_energies_lists[potential])
            interaction_info.extend([native_energies, partial_energies, external_energies])
        if not Zscores:
            return interaction_info
        # Generate random fold energies
        rand_energies_lists = {}
        for random in xrange(randoms):
            rand_interface_R = self.randomize_residue_interface(interaction.get_structure1().get_residues(), real_interactions[0])
            rand_interface_L = self.randomize_residue_interface(interaction.get_structure2().get_residues(), real_interactions[1])
            rand_energies = self.calculate_energies(zip(rand_interface_R, rand_interface_L, real_interactions[2], real_interactions[0], real_interactions[1]), randomized=True)
            for potential_type in rand_energies:
                rand_energies_lists.setdefault(potential_type, []).append(rand_energies[potential_type])
        # Compute Zscores
        real_Zenergies = self.compute_Zscores(real_energies, rand_energies_lists)
        interaction_info.append(real_Zenergies)
        if funnel:
            native_Zenergies = self.compute_Zscores(native_energies, rand_energies_lists)
            partial_Zenergies = self.compute_Zscores(partial_energies, rand_energies_lists)
            external_Zenergies = self.compute_Zscores(external_energies, rand_energies_lists)
            interaction_info.extend([native_Zenergies, partial_Zenergies, external_Zenergies])
        return interaction_info

    def calculate_residue_energies(self, interaction, struc, Zscores=False, randoms=100, alanine_scanning=False, funnel=False):
        '''
        Compute the split potential energies for each interacting residue in a protein-protein interaction
        if Zscores: Computes also the Zscores
        if alanine_scanning: Computes also the alanine mutation of each residue
        if funnel: now the residue energy is the mean of the interaction with the whole protein (random noninterating exposed residues)
        '''
        struc_accepted = ('R','L') # Receptor or Ligand residue energies
        if struc.upper() not in struc_accepted:
            sys.stderr.write('WARNING: Incorrect structure type %s, using R (receptor)!' % struc)
        # Compute energies for each residue
        real_interactions = interaction.get_interacting_residues(c_type=self.c_type, max_distance=self.cutoff+1, uniq=False)
        if struc == 'L':
            interacting_residues = list(set(real_interactions[1]))
            structureA = interaction.get_structure2()
            structureB = interaction.get_structure1()
            noninterating_exposedB = self.__get_noninteracting_residues(list(set(real_interactions[0])), structureB)
        else:
            interacting_residues = list(set(real_interactions[0]))
            structureA = interaction.get_structure1()
            structureB = interaction.get_structure2()
            noninterating_exposedB = self.__get_noninteracting_residues(list(set(real_interactions[1])), structureB)
        residues_interactions = []
        for interacting_residue in interacting_residues:
            residue_interactions = []
            for i in xrange(len(real_interactions[2])):
                if struc == 'L':
                    if real_interactions[1][i] == interacting_residue:
                        residue_interactions.append((real_interactions[1][i], real_interactions[0][i], real_interactions[2][i]))
                else:
                    if real_interactions[0][i] == interacting_residue:
                        residue_interactions.append((real_interactions[0][i], real_interactions[1][i], real_interactions[2][i]))
            residue_energies = self.calculate_energies(residue_interactions, local_type='R')
            residue_info = [interacting_residue, residue_interactions, residue_energies]
            if alanine_scanning:
                alanine_energies = self.calculate_energies(residue_interactions, local_type='R', alanine_mutation=True)
                residue_info.append(alanine_energies)
            if funnel:
                rand_funn_energies_lists = {}
                funnel_energies = {}
                if alanine_scanning:
                    rand_fala_energies_lists = {}
                    funnel_ala_energies = {}
                for random in xrange(randoms):
                    interfaceA, interfaceB, distances = zip(*residue_interactions)
                    rand_interfaceB = self.randomize_residue_interface(noninterating_exposedB, interfaceB)
                    rand_funn_energies = self.calculate_energies(zip(interfaceA, rand_interfaceB, distances, interfaceA, interfaceB), local_type='R', randomized=True)
                    for potential_type in rand_funn_energies:
                        rand_funn_energies_lists.setdefault(potential_type, []).append(rand_funn_energies[potential_type])  
                    if alanine_scanning:
                        rand_fala_energies = self.calculate_energies(zip(interfaceA, rand_interfaceB, distances, interfaceA, interfaceB), local_type='R', randomized=True, alanine_mutation=True)
                        for potential_type in rand_fala_energies:
                            rand_fala_energies_lists.setdefault(potential_type, []).append(rand_fala_energies[potential_type]) 
                for potential in rand_funn_energies_lists:
                    funnel_energies[potential] = numpy.mean(rand_funn_energies_lists[potential])
                residue_info.append(funnel_energies)
                if alanine_scanning:
                    for potential in rand_fala_energies_lists:
                        funnel_ala_energies[potential] = numpy.mean(rand_fala_energies_lists[potential])                  
                    residue_info.append(funnel_ala_energies)
            residues_interactions.append(residue_info)
        if not Zscores:
            return residues_interactions
        # Generate Zscore energies for each residues
        residues_energies = []
        for residue in residues_interactions:
            rand_energies_lists = {}
            for random in xrange(randoms):
                interfaceA, interfaceB, distances = zip(*residue[1])
                rand_interfaceA = self.randomize_residue_interface(structureA.get_residues(), interfaceA)
                rand_interfaceB = self.randomize_residue_interface(structureB.get_residues(), interfaceB)
                rand_energies = self.calculate_energies(zip(rand_interfaceA, rand_interfaceB, distances, interfaceA, interfaceB), local_type='R', randomized=True)
                for potential_type in rand_energies:
                    rand_energies_lists.setdefault(potential_type, []).append(rand_energies[potential_type])
            residue_info = [residue[0], residue[2], self.compute_Zscores(residue[2], rand_energies_lists)]
            info_index = 3
            if alanine_scanning:
                residue_info.extend([residue[info_index], self.compute_Zscores(residue[info_index], rand_energies_lists)])
                info_index += 1
            if funnel:
                residue_info.extend([residue[info_index], self.compute_Zscores(residue[info_index], rand_energies_lists)])
                info_index += 1
            if alanine_scanning and funnel:
                residue_info.extend([residue[info_index], self.compute_Zscores(residue[info_index], rand_energies_lists)])
            residue_info.append(len(residue[1]))
            residues_energies.append(residue_info)
        return residues_energies

    ##################
    # PRIVATE METHODS #
    ##################

    def __get_noninteracting_residues(self, int_residues, structure, exposed=True):
        '''
        Return all exposed non-interacting residues
        '''
        out_residues = []
        for residue in structure:
            if exposed:
                try:
                    residue_exposed = residue.is_exposed()
                except:
                    continue
                if not residue_exposed:
                    continue
            if residue not in int_residues:
                out_residues.append(residue)
        return out_residues

    ########################################################
    # METHODS TO COMPUTE SPLIT POTENTIALS FOR INTERACTIONS #
    ########################################################

    def calculate_energies(self, res_interactions, local_type='G', randomized=False, alanine_mutation=False):
        '''
        Compute the split potential energies for a set of residue-residue interactions
        '''
        local_type_accepted = ('G','R') # Global or Residue calculations
        if local_type.upper() not in local_type_accepted:
            sys.stderr.write('WARNING: Incorrect local type %s, using G!' % local_type)
            local_type = 'G'

        # Init Energy values
        energies = {'D-PAIR':   0, 'C-PAIR':  0, 'U-PAIR':  0, 'L-PAIR':  0, 'M-PAIR':  0,
                    'D-3DC':    0, 'C-3DC':   0, 'U-3DC':   0, 'L-3DC':   0, 'M-3DC':   0,
                    'D-LOCAL':  0,
                    'D-S3DC':   0, 'C-S3DC':  0, 'U-S3DC':  0, 'L-S3DC':  0, 'M-S3DC':  0,
                    'D-3D':     0, 'C-3D':    0,
                    'D-COMB':   0, 'C-COMB':  0, 'U-COMB':  0, 'L-COMB':  0, 'M-COMB':  0}

        # Compute energy for each residue interaction
        for res_interaction in res_interactions:
            distance = res_interaction[2]
            try:
                triad1 = res_interaction[0].get_triad()
                triad2 = res_interaction[1].get_triad()
                if randomized:
                    triad1 = res_interaction[0].get_triad()[:4] + res_interaction[3].get_triad()[4:]
                    triad2 = res_interaction[1].get_triad()[:4] + res_interaction[4].get_triad()[4:]
                if alanine_mutation: # Generally with local_type R
                    triad1 = 'A-n-' + res_interaction[0].get_triad()[4:]
            except ResidueTriadError as e:
                #sys.stdout.write(str(e))
                #sys.stdout.flush()
                continue

            # Energy of Pair Potentials of Residues #
            distance_ppR           = self.get_ppR(triad1[0:1], triad2[0:1], distance)
            energies['D-PAIR']    += distance_ppR
            cutoff_ppR             = self.get_ppR(triad1[0:1], triad2[0:1], self.cutoff)
            energies['C-PAIR']    += cutoff_ppR
            maximum_ppR            = self.get_ppR_stat(triad1[0:1], triad2[0:1], stat='upper')
            energies['U-PAIR']    += maximum_ppR
            minimum_ppR            = self.get_ppR_stat(triad1[0:1], triad2[0:1], stat='lower')
            energies['L-PAIR']    += minimum_ppR
            mean_ppR               = self.get_ppR_stat(triad1[0:1], triad2[0:1], stat='mean')
            energies['M-PAIR']    += mean_ppR
            # Energy of Pair Potentials of Environment
            distance_ppE           = self.get_ppE(triad1[2:], triad2[2:], distance)
            energies['D-3DC']     += distance_ppE
            cutoff_ppE             = self.get_ppE(triad1[2:], triad2[2:], self.cutoff)
            energies['C-3DC']     += cutoff_ppE
            maximum_ppE            = self.get_ppE_stat(triad1[2:], triad2[2:], stat='upper')
            energies['U-3DC']     += maximum_ppE
            minimum_ppE            = self.get_ppE_stat(triad1[2:], triad2[2:], stat='lower')
            energies['L-3DC']     += minimum_ppE
            mean_ppE               = self.get_ppE_stat(triad1[2:], triad2[2:], stat='mean')
            energies['M-3DC']     += mean_ppE
            # Energy of Local Pair Potentials of Residues and Environment
            if local_type == 'R':
                distance_LppRE     = self.get_LppRE(triad1)
            else:
                distance_LppRE     = self.get_LppRE(triad1) + self.get_LppRE(triad2)
            energies['D-LOCAL']   += distance_LppRE
            # Energy of Pair Potentials of Residues and Environment
            distance_ppRE          = self.get_ppRE(triad1, triad2, distance)
            energies['D-S3DC']    += distance_ppRE
            cutoff_ppRE            = self.get_ppRE(triad1, triad2, self.cutoff)
            energies['C-S3DC']    += cutoff_ppRE
            maximum_ppRE           = self.get_ppRE_stat(triad1, triad2, stat='upper')
            energies['U-S3DC']    += maximum_ppRE
            minimum_ppRE           = self.get_ppRE_stat(triad1, triad2, stat='lower')
            energies['L-S3DC']    += minimum_ppRE
            mean_ppRE              = self.get_ppRE_stat(triad1, triad2, stat='mean')
            energies['M-S3DC']    += mean_ppRE
            # Energy of Standard Pair Potentials of Distances
            distance_ppD           = self.get_ppD(distance)
            energies['D-3D']      += distance_ppD
            cutoff_ppD             = self.get_ppD(self.cutoff)
            energies['C-3D']      += cutoff_ppD
            # Energy of Combined energy
            distance_comb          = distance_ppRE + distance_ppD + distance_LppRE + distance_ppE
            energies['D-COMB']  += distance_comb
            cutoff_comb            = cutoff_ppRE + cutoff_ppD + distance_LppRE + cutoff_ppE
            energies['C-COMB']  += cutoff_comb
            maximum_comb           = maximum_ppRE + distance_ppD + distance_LppRE + maximum_ppE
            energies['U-COMB']  += maximum_comb
            minimum_comb           = minimum_ppRE + distance_ppD + distance_LppRE + minimum_ppE
            energies['L-COMB']  += minimum_comb
            mean_comb              = mean_ppRE + distance_ppD + distance_LppRE + mean_ppE
            energies['M-COMB']  += mean_comb

        return energies

    ##################################
    # CONTACT RANDOMIZATION  METHODS #
    ##################################

    def randomize_residue_interface(self, struct_residues, res_interface):

        rand_interface = []
        done = {}
        for res in res_interface:
            if done.has_key(res):
                rand_res = done[res]
            else:
                rand_res = random.choice(struct_residues)
                done[res] = rand_res
            rand_interface.append(rand_res)
        return rand_interface

    def compute_Zscores(self, real_energies, rand_energies_lists):

        Zenergies = {} 
        for potential_type in real_energies:
            if potential_type == 'D-3D' or potential_type == 'C-3D':
                continue
            energies_mean = numpy.mean(rand_energies_lists[potential_type])
            energies_std  = numpy.std(rand_energies_lists[potential_type])
            Zenergies[potential_type] = (real_energies[potential_type]-energies_mean)/energies_std
        return Zenergies

    ############################
    # SPLIT POTENTIALS GETTERS #
    ############################

    def get_ppR(self, res1, res2, distance):

        distance = int(math.floor(distance))
        if self.ppR.has_key(res1) and self.ppR[res1].has_key(res2):
            return float(self.ppR[res1][res2][distance])
        elif self.ppR.has_key(res2) and self.ppR[res2].has_key(res1):
            return float(self.ppR[res2][res1][distance])
        else:
            return 0
            #raise RuntimeError('ERROR: Matrix entry not found: %s, %s' % (res1, res2))
        
    def get_ppR_stat(self, res1, res2, stat):

        if self.ppR.has_key(res1) and self.ppR[res1].has_key(res2):
            data = numpy.array(self.ppR[res1][res2], dtype='float64')
        elif self.ppR.has_key(res2) and self.ppR[res2].has_key(res1):
            data = numpy.array(self.ppR[res2][res1], dtype='float64')
        else:
            return 0
            #raise RuntimeError('ERROR: Matrix entry not found: %s, %s, %d' % (res1, res2))
        if stat.upper() == 'UPPER':
            return numpy.max(data)
        elif stat.upper() == 'LOWER':
            return numpy.min(data)
        elif stat.upper() == 'MEAN':
            return numpy.mean(data)
        
    def get_ppE(self, res1, res2, distance):

        distance = int(math.floor(distance))
        if self.ppE.has_key(res1) and self.ppE[res1].has_key(res2):
            return -float(self.ppE[res1][res2][distance])
        elif self.ppE.has_key(res2) and self.ppE[res2].has_key(res1):
            return -float(self.ppE[res2][res1][distance])
        else:
            return 0
            #raise RuntimeError('ERROR: Matrix entry not found: %s, %s' % (res1, res2))

    def get_ppE_stat(self, res1, res2, stat):

        if self.ppE.has_key(res1) and self.ppE[res1].has_key(res2):
            data = numpy.array(self.ppE[res1][res2], dtype='float64')
        elif self.ppE.has_key(res2) and self.ppE[res2].has_key(res1):
            data = numpy.array(self.ppE[res2][res1], dtype='float64')
        else:
            return 0
            #raise RuntimeError('ERROR: Matrix entry not found: %s, %s' % (res1, res2))
        # UPPER and LOWER are inverted in this potential
        if stat.upper() == 'UPPER':
            return -numpy.min(data)
        elif stat.upper() == 'LOWER':
            return -numpy.max(data)
        elif stat.upper() == 'MEAN':
            return -numpy.mean(data)
              
    def get_LppRE(self, res):

        return -float(self.LppRE[res])
    
    def get_ppRE(self, res1, res2, distance):
        '''
        '''
        distance = int(math.floor(distance))
        if self.ppRE.has_key(res1) and self.ppRE[res1].has_key(res2):
            return float(self.ppRE[res1][res2][distance])
        elif self.ppRE.has_key(res2) and self.ppRE[res2].has_key(res1):
            return float(self.ppRE[res2][res1][distance])
        else:
            return 0
            #raise RuntimeError('ERROR: Matrix entry not found: %s, %s' % (res1, res2))

    def get_ppRE_stat(self, res1, res2, stat):
        '''
        '''
        if self.ppRE.has_key(res1) and self.ppRE[res1].has_key(res2):
            data = numpy.array(self.ppRE[res1][res2], dtype='float64')
        elif self.ppRE.has_key(res2) and self.ppRE[res2].has_key(res1):
            data = numpy.array(self.ppRE[res2][res1], dtype='float64')
        else:
            return 0
            #raise RuntimeError('ERROR: Matrix entry not found: %s, %s' % (res1, res2))
        if stat.upper() == 'UPPER':
            return numpy.max(data)
        elif stat.upper() == 'LOWER':
            return numpy.min(data)
        elif stat.upper() == 'MEAN':
            return numpy.mean(data)
        
    def get_ppD(self, distance):
        '''
        '''
        distance = str(int(math.floor(distance)))
        return float(self.ppD[distance])

    #########################################################
    # METHODS TO READ AND PROCESS THE SPLIT POTENTIALS FILE #
    #########################################################

    def read_sp_file(self):

        tag = 0
        sp_file_fo = open(self.sp_file, 'r')
        for line in sp_file_fo:
            if line.startswith("PAIR-POTENTIALS OF RESIDUES AND ENVIRONMENTS"):             tag = 4
            elif line.startswith("PAIR-POTENTIALS OF ENVIRONMENTS"):                        tag = 2
            elif line.startswith("LOCAL PAIR-POTENTIALS OF RESIDUES AND ENVIRONMENTS"):     tag = 3
            elif line.startswith("PAIR-POTENTIALS OF RESIDUES"):                            tag = 1
            elif line.startswith("STANDARD PAIR-POTENTIALS OF DISTANCES"):                  tag = 5
            elif line.startswith("END"):                                                    tag = 0
            self.process_sp_file_line(line, tag)
        sp_file_fo.close()

    def process_sp_file_line(self, line, tag):

        if tag == 0:    
            return
        if tag == 1 and line.startswith(" PAIR"): 
            self.assign_multiple_distances(self.ppR, line.split()[1], line.split()[2:])
        if tag == 2 and line.startswith(" PAIR"): 
            self.assign_multiple_distances(self.ppE, line.split()[1], line.split()[2:])
        if tag == 3 and line.startswith(" AA-ENV"): 
            self.assign_single_distance(self.LppRE, line.split()[1], line.split()[2])
        if tag == 4 and line.startswith(" PAIR"): 
            self.assign_multiple_distances(self.ppRE, line.split()[1], line.split()[2:])
        if tag == 5 and line.startswith(" Distance"): 
            self.assign_single_distance(self.ppD, line.split()[1], line.split()[2])

    def assign_multiple_distances(self, matrix, keys, values):
        
        matrix.setdefault(keys.split(':')[0],{}).setdefault(keys.split(':')[1],[]).extend(values)
        
    def assign_single_distance(self, matrix, key, value):
        
        matrix[key] = value

    ##################################################
    # METHODS TO PRINT THE SPLIT POTENTIALS MATRICES #
    ##################################################

    def two_keys_matrix_2_string_array(self, matrix, prev):
        
        lines = []
        for res1 in matrix:
            for res2 in matrix[res1]:
                line = prev + "%s:%s\t" % (res1, res2)
                line = line + "\t".join(matrix[res1][res2])
                lines.append(line)
        return lines
                
    def one_key_matrix_2_string_array(self, matrix, prev):
        
        lines = []
        for k in matrix:
            line = prev + "\t".join((k,matrix[k]))
            lines.append(line)
        return lines

    def ppR_2_string_array(self):
        
        lines = []
        lines.append("PAIR-POTENTIALS OF RESIDUES")
        lines.extend(self.two_keys_matrix_2_string_array(self.ppR, " PAIR "))
        lines.append("END")
        return lines
    
    def string_ppR(self): 

        return "\n".join(self.ppR_2_string_array())

    def ppE_2_string_array(self):
        
        lines = []
        lines.append("PAIR-POTENTIALS OF ENVIRONMENTS")
        lines.extend(self.two_keys_matrix_2_string_array(self.ppE, " PAIR "))
        lines.append("END")
        return lines
    
    def string_ppE(self): 

        return "\n".join(self.ppE_2_string_array())
    
    def LppRE_2_string_array(self):
        
        lines = []
        lines.append("LOCAL PAIR-POTENTIALS OF RESIDUES AND ENVIRONMENTS")
        lines.extend(self.one_key_matrix_2_string_array(self.LppRE, " AA-ENV "))
        lines.append("END")
        return lines
    
    def string_LppRE(self): 

        return "\n".join(self.LppRE_2_string_array())

    def ppRE_2_string_array(self):
        
        lines = []
        lines.append("PAIR-POTENTIALS OF RESIDUES AND ENVIRONMENTS")
        lines.extend(self.two_keys_matrix_2_string_array(self.ppRE, " PAIR "))
        lines.append("END")
        return lines
    
    def string_ppRE(self): 

        return "\n".join(self.ppRE_2_string_array())

    def ppD_2_string_array(self):
        
        lines = []
        lines.append("STANDARD PAIR-POTENTIALS OF DISTANCES")
        lines.extend(self.one_key_matrix_2_string_array(self.ppD, " Distance "))
        lines.append("END")
        return lines
    
    def string_ppD(self): 

        return "\n".join(self.ppD_2_string_array())

    def __str__(self):
        
        lines = []
        lines.extend(self.ppR_2string_array())
        lines.extend(self.ppE_2string_array())
        lines.extend(self.LppRE_2string_array())
        lines.extend(self.ppRE_2string_array())
        lines.extend(self.ppD_2string_array())
        return "\n".join(lines)
        


import json
from abc import ABCMeta, abstractmethod


class JSONer(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def as_dict(self):
        pass

    def json(self, pretty=False):
        if pretty:
            return json.dumps(self.as_dict(), indent=2, separators=(',', ':'))
        return json.dumps(self.as_dict(), separators=(',', ':'))



import sys
import os
import argparse
import gzip

from SBI.beans           import Path
from SBI.structure.protein import Arch as SSpair

from archdb.src import Source, Arch
from archdb.src import start_transaction, end_transaction


# User Options
def options(*args, **kwds):
    parser = argparse.ArgumentParser(prog = 'Arch2SQL',  formatter_class = argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-d', '--database',     dest     = "database",
                        action = "store",       required = True,  help = "Arch output dir.")
    parser.add_argument('-s', '--sql',          dest     = "sqlfile",
                        action = "store",       required = True,  help = "Output SQL dir")
    parser.add_argument('-v', '--verbose',      dest     = "verbose",
                        action = "store_true",  default  = False, help = "Verbose Mode.")
    options = parser.parse_args()
    return options


def Arch2SQL(database, sqlfile, verbose):

    if verbose:
        sys.stderr.write("Retrieving data from {0} ...\n".format(database))
    newsource = Source(name = 'PDBarch', source = "http://www-pdb.org/")
    outdir = os.path.join(os.path.join(os.path.abspath(sqlfile), '00'))
    Path.mkdir(outdir)
    sql_fd = gzip.open(os.path.join(outdir, '0000.sql.gz'), 'wb')
    sql_fd.write(start_transaction())
    sql_fd.write(newsource.toSQL())
    sql_fd.write(end_transaction())
    sql_fd.close()

    files_list_by_pdb = {}
    subdirs = ['archobj', 'superobj']
    for subdir in subdirs:
        for archobjfile in Path.list_files(os.path.join(database, subdir)):
            if archobjfile.endswith('.archObj'):
                data = tuple(os.path.splitext(os.path.split(archobjfile)[-1])[0].split('_')[2:])
                files_list_by_pdb[data] = archobjfile

    old_pdb = None
    newArchSet = None
    for dofdata in sorted(files_list_by_pdb):
        pdb = dofdata[0] + '_' + dofdata[1]
        if pdb != old_pdb:
            if old_pdb is not None:
                sql_fd.write(newArchSet.toSQL())
                sql_fd.write(end_transaction())
                sql_fd.close()
            outdir = os.path.join(os.path.join(os.path.abspath(sqlfile), dofdata[0][1:3].lower()))
            Path.mkdir(outdir)
            if verbose:
                sys.stderr.write("Retrieving loops from {0} ...\n".format(pdb))
            sql_fd = gzip.open(os.path.join(outdir, pdb + '.sql.gz'), 'wb')
            sql_fd.write(start_transaction())
            if verbose:
                sys.stderr.write("Printing data from {0} ...\n".format(pdb))
            old_pdb    = pdb
            newArchSet = Arch(pdb)
        newArchSet.archs = SSpair.load(files_list_by_pdb[dofdata])

    sql_fd.write(newArchSet.toSQL())
    sql_fd.write(end_transaction())
    sql_fd.close()
    if verbose:
        sys.stderr.write("End execution.\n")

if __name__ == "__main__":
    options = options()
    Arch2SQL(options.database, options.sqlfile, options.verbose)



'''
jbonet @ boliva's lab 2013
'''

import os
import sys
import ConfigParser

from SBI.beans.Executable import Executable
from DSSP                 import DSSP       as DSSP


class DSSPExe(object):

    default_config_file = os.path.join(
        os.path.normpath(
            os.path.join(
                os.path.dirname(__file__), '..')),
        'configSBI.txt')

    #DSSP executable configuration
    _config = ConfigParser.RawConfigParser(allow_no_value=True)
    _config.read(os.getenv('SBI_CONFIG_FILE', default_config_file))
    _exe = None

    def __init__(self, pdbfile, dsspfile, cleanpdb=False, cleandssp=False):

        self._pdbfile  = pdbfile
        self._dsspfile = dsspfile
        self._dsspdata = []
        self._gapped   = False

        if DSSPExe._exe is None:
            DSSPExe._exe = Executable(
                executable=DSSPExe._config.get('dssp', 'executable'),
                path=DSSPExe._config.get('dssp', 'path'))

        self._execute()
        self._parse()
        self._clean(cleanpdb, cleandssp)

    @property
    def dsspdata(self):
        return self._dsspdata

    @property
    def gapped(self):
        return self._gapped

    @property
    def empty_dssp(self):
        return DSSP(secondary_structure = '-',
                    accessibility       = 1,
                    AAtype              = 'X')

    def _execute(self):
        self._exe.add_parameter(self._pdbfile)
        self._exe.add_parameter(self._dsspfile)
        try:
            self._exe.execute(silent=True)
        except SystemError, e:
            msg = 'Some error occurred while executing dssp\n{0}\n'.format(e)
            sys.stderr.write(msg)

        self._exe.clean_command()

    @staticmethod
    def dynamic(executable, path):
        DSSPExe._exe = Executable(executable=executable, path=path)

    def _parse(self):
        file_fd    = open(self._dsspfile)
        read       = False
        continuity = -1000
        readline   = 0
        for line in file_fd:
            if line.startswith("  #  RESIDUE AA STRUCTURE BP1 BP2  ACC"):
                read = True
                continue
            if read:
                if line[13:14] != '!':
                    res_num = int(line[6:10].strip())
                    ss      = line[16:17]
                    buried  = int(line[35:38].strip())
                    aa      = line[13:15].strip()
                    if ss == " ":
                        ss = "-"
                    self._dsspdata.append(DSSP(secondary_structure = ss,
                                               accessibility       = buried,
                                               AAtype              = aa))
                    self._dsspdata[-1].add_hydrogen_links(line[39:50],
                                                          line[50:61],
                                                          line[61:72],
                                                          line[72:84])
                    if readline > 0:
                        if res_num != continuity + 1:
                            self._gapped = True
                        continuity = res_num
                    readline += 1
                else:
                    msg = "truncated chain!{0}\n".format(self._dsspfile)
                    sys.stderr.write(msg)
                    self._gapped = True
        file_fd.close()

    def _clean(self, cleanpdb, cleandssp):
        if cleanpdb:
            os.unlink(self._pdbfile)
        if cleandssp:
            os.unlink(self._dsspfile)



# from SBI.structure import PDB, ResidueOfNucleotide, ResidueOfAminoAcid
# from SBI.contacts  import PPInterface, PNInterface

from SBI.structure           import PDB
from SBI.structure.contacts  import Complex, InnerContacts
from SBI                         import SBIglobals

tables = {'main'        : 'PDB',
          'old'         : 'oldPDB',
          'chain'       : 'chain',
          'ec'          : 'chain2enzyme',
          'enz'         : 'enzyme',
          'ctaxid'      : 'chain2taxid',
          'taxid'       : 'taxid',
          'taxidold'    : 'taxid_old',
          'uniprot'     : 'uniprot',
          'pdbuniprot'  : 'chain2uniprot',
          'ent2acc'     : 'uniprot_entry2accession',
          'hetatm'      : 'hetero',
          'hetchn'      : 'hetero2PDB',
          'repeidx'     : 'chainrepeatidx',
          'repe'        : 'chainrepeat',
          'site'        : 'site',
          'sitepos'     : 'site2position',
          'h6c'         : 'hetero_contacts6',
          'cont12'      : 'contacts12',
          'chainc12'    : 'chain_contacts12'}

class InnerContacts(InnerContacts):
    def toSQL(self):
        SBIglobals.alert('debug', self, 'Printing Inner Contacts (Protein-Heteroatom) as ArchDB SQL')
        command = ''
        for ht in self.HTcontacts:
            if len(ht) == 0: continue
            SBIglobals.alert('debug', self, '\tInner Contact {0} for chain {1}'.format(ht.identifier, ht.protein.chain))
            command += "SET @chain = (SELECT nid FROM {0} WHERE pdb='{1}' AND chain='{2}');\n".format(tables['chain'], self.pdb.id, ht.protein.chain)
            for hetero in ht.heteroatoms_view_innercontact:
                SBIglobals.alert('debug', self, '\tFound {0} contacts for HETATM {1.type} in {1.number}'.format(len(ht.heteroatoms_view_innercontact[hetero]), hetero))
                command += "SET @heteroID = gethetero2PDB(@chain,{0.number},'{0.type}');\n".format(hetero)
                for contact in ht.heteroatoms_view_innercontact[hetero]:
                    command += "INSERT INTO {0}(chain,hetero,position,idxp,type,distance) VALUES (@chain,@heteroID,{2.number},'{2.version}','{2.type}',{1:.3f});\n".format(tables['h6c'], contact.min_distance,contact.aminoacid)
        return command

class Complex(Complex):
    def toSQL(self):
        SBIglobals.alert('debug', self, 'Printing Interfaces as ArchDB SQL')
        command = ''
        for ppi in self.PPInterfaces:
            if len(ppi) == 0: continue
            SBIglobals.alert('debug', self, '\tProtein - Protein Interface {0} for chains {1} - {2}'.format(ppi.identifier, ppi.protein_chain.chain, ppi.protein_interactor.chain))
            command += "SET @chainA = (SELECT nid FROM {0} WHERE pdb='{1}' AND chain='{2}');\n".format(tables['chain'], self.pdb.id, ppi.protein_chain.chain)
            command += "SET @chainB = (SELECT nid FROM {0} WHERE pdb='{1}' AND chain='{2}');\n".format(tables['chain'], self.pdb.id, ppi.protein_interactor.chain)
            SBIglobals.alert('debug', self, '\tFound a total of {0} contacts'.format(len(ppi.contacts)))
            for contact in ppi.contacts:
                command += "INSERT INTO {0} (alpha_distance, beta_distance, min_distance, type) VALUES ({1:.3f},{2:.3f},{3:.3f},'PPI');\n".format(tables['cont12'], contact.ca_distance, contact.cb_distance, contact.min_distance)
                command += "SET @contact = LAST_INSERT_ID();\n";
                command += "INSERT INTO {0} (contacts12, chain, position, idxp, type) VALUES (@contact,@chainA,{1.number},'{1.version}','{1.type}');\n".format(tables['chainc12'], contact.aminoacid1)
                command += "INSERT INTO {0} (contacts12, chain, position, idxp, type) VALUES (@contact,@chainB,{1.number},'{1.version}','{1.type}');\n".format(tables['chainc12'], contact.aminoacid2)

        for pni in self.PNInterfaces:
            if len(pni) == 0: continue
            SBIglobals.alert('debug', self, '\tProtein - Nucleotide Interface {0} for chains {1} - {2}'.format(pni.identifier, pni.protein.chain, pni.nucleotide.chain))
            command += "SET @chainP = (SELECT nid FROM {0} WHERE pdb='{1}' AND chain='{2}');\n".format(tables['chain'], self.pdb.id, pni.protein.chain)
            command += "SET @chainN = (SELECT nid FROM {0} WHERE pdb='{1}' AND chain='{2}');\n".format(tables['chain'], self.pdb.id, pni.nucleotide.chain)
            SBIglobals.alert('debug', self, '\tFound a total of {0} contacts'.format(len(pni.contacts)))
            for contact in pni.contacts:
                command += "INSERT INTO {0} (beta_distance, min_distance, type) VALUES ({1:.3f},{2:.3f},'PNI');\n".format(tables['cont12'], contact.cbbackbone_distance, contact.min_distance)
                command += "SET @contact = LAST_INSERT_ID();\n";
                command += "INSERT INTO {0} (contacts12, chain, position, idxp, type) VALUES (@contact,@chainP,{1.number},'{1.version}','{1.type}');\n".format(tables['chainc12'], contact.aminoacid)
                command += "INSERT INTO {0} (contacts12, chain, position, idxp, type) VALUES (@contact,@chainN,{1.number},'{1.version}','{1.type}');\n".format(tables['chainc12'], contact.nucleotide)
        for phi in self.PHInterfaces:
            if len(phi) == 0: continue
            SBIglobals.alert('debug', self, '\tProtein - Heteroatom Interface {0} for chains {1} - {2}'.format(phi.identifier, phi.protein.chain, phi.heteroatoms.chain))
            command += "SET @chainP = (SELECT nid FROM {0} WHERE pdb='{1}' AND chain='{2}');\n".format(tables['chain'], self.pdb.id, phi.protein.chain)
            command += "SET @chainH = (SELECT nid FROM {0} WHERE pdb='{1}' AND chain='{2}');\n".format(tables['chain'], self.pdb.id, phi.heteroatoms.chain)
            for hetero in phi.heteroatoms_view_interface:
                SBIglobals.alert('debug', self, '\tFound {0} contacts for HETATM {1.type} in {1.number} on {2} with chain {3}'.format(len(phi.heteroatoms_view_interface[hetero]), hetero, phi.heteroatoms.chain, phi.protein.chain))
                command += "SET @heteroID = gethetero2PDB(@chainH,{0.number},'{0.type}');\n".format(hetero)
                for contact in phi.heteroatoms_view_interface[hetero]:
                    command += "INSERT IGNORE INTO {0}(chain,hetero,position,idxp,type,distance) VALUES (@chainP,@heteroID,{2.number},'{2.version}','{2.type}',{1:.3f});\n".format(tables['h6c'], contact.min_distance,contact.aminoacid)
        return command

class PDB(PDB):
    def __init__(self, pdb_file = None):
        SBIglobals.alert('debug', self, 'Loading PDB file {0}'.format(pdb_file))

        super(PDB, self).__init__(pdb_file = pdb_file, dehydrate = True, header = True)

        SBIglobals.alert('debug', self, 'Calculating Inner Contacts for Protein - Heteroatom')
        self.innercontacts = InnerContacts(pdb = self, AA = False,
                                                       NC = False,
                                                       HT = True,  HT_type = "min", HT_distance = 6)

        SBIglobals.alert('debug', self, 'Calculating PP and PN interfaces of the biomolecules in the PDB')
        self.interfaces    = Complex(pdb = self, biomolecule = True, PPI = True, PPI_type = "cb",  PPI_distance = 12,
                                                                     PNI = True, PNI_type = "min", PNI_distance = 8,
                                                                     PHI = True, PHI_type = "min", PHI_distance = 6)

    @staticmethod
    def preuniprotdeleted():
        return "INSERT INTO {0} (entry, source) VALUES ('FAKE_PROTEIN', 'swissprot');\n".format(tables['uniprot'])
    @staticmethod
    def afteruniprotdeleted():
        command  = "DELETE FROM {0} WHERE uniprot = 'FAKE_PROTEIN';\n".format(tables['pdbuniprot'])
        command += "DELETE FROM {0} WHERE entry = 'FAKE_PROTEIN';\n".format(tables['uniprot'])
        return command

    @staticmethod
    def _choose_ec(ec):
        command = []
        data = ec.split('.')
        main = "(SELECT id FROM {0} WHERE id = '{1}')"
        if len(data) == 3: data.append('-')
        command.append(main.format(tables['enz'], '.'.join(data)))
        if data[3] != '-':
            data[3] = '-'
            command.append(main.format(tables['enz'], '.'.join(data)))
        if data[2] != '-':
            data[2] = '-'
            command.append(main.format(tables['enz'], '.'.join(data)))
        if data[1] != '-':
            data[1] = '-'
            command.append(main.format(tables['enz'], '.'.join(data)))
        return 'SET @ec = (COALESCE('+ ",".join(command)+'));\n'

    def toSQL(self):
        pdbheader = self.header

        #if pdbheader.valid_resolution and pdbheader.resolution != 'NULL':
        if pdbheader.experiment.resolution >0 :
            command = "INSERT INTO {0} VALUES ('{1.id}','{2.date}','{2.header}','{2.xpdta}',{2.resolution:.2f},{2.rfactor:.3f},{2.freeR:.3f});\n".format(tables['main'], self, pdbheader)
        else:
            command = "INSERT INTO {0} (pdb,date,header,method) VALUES ('{1.id}','{2.date}','{2.header}','{2.xpdta}');\n".format(tables['main'], self, pdbheader)
        for deprec in pdbheader.deprecated:
            command += "INSERT INTO {0} VALUES ('{1}','{2.id}');\n".format(tables['old'], deprec, self)
        for chain in self.chains:
            m = pdbheader.get_molecule4chain(chain.chain)
            f = chain.first_structure
            l = chain.last_structure
            command += "INSERT INTO {0}(pdb,chain,name,type,start,idxs,end,idxe) VALUES ('{1.pdb}','{1.chain}','{2.name}','{1.chaintype}',{3.number},'{3.version}',{4.number},'{4.version}');\n".format(tables['chain'],chain,m,f,l)
            command += "SET @chain = LAST_INSERT_ID();\n";
            for ec in m.ec:
                command += PDB._choose_ec(ec)
                command += "INSERT INTO {0} VALUES (@chain,@ec);\n".format(tables['ec'])
            for tx in m.taxid:
                command += "SET @taxid = (SELECT COALESCE((SELECT taxid FROM {0} WHERE oldid={2}),(SELECT id FROM {1} WHERE id={2})));\n".format(tables['taxidold'], tables['taxid'], tx)
                command += "INSERT INTO {0} VALUES (@chain,@taxid);\n".format(tables['ctaxid'])
        for dbr in pdbheader.dbrefs:
            if dbr.chain in self._chain_id:
                command += "SET @chain = (SELECT nid FROM {0} WHERE pdb='{1}' AND chain='{2}');\n".format(tables['chain'], self.id, dbr.chain)
                command += "SET @uniprot = (COALESCE((SELECT entry FROM {0} WHERE accession='{1}' LIMIT 1), 'FAKE_PROTEIN'));\n".format(tables['ent2acc'], dbr.uniprot)
                command += "INSERT INTO {0} VALUES (@chain,@uniprot,{1.start},'{1.idxs}',{1.end},'{1.idxe}');\n".format(tables['pdbuniprot'], dbr)
        for h in pdbheader.hetero:
            if h.chain in self._chain_id:
                command += "INSERT IGNORE INTO {0} VALUES ('{1.id}','{1._name}','{1.form}', 0, 0);\n".format(tables['hetatm'], h)
                command += "SET @chain = (SELECT nid FROM {0} WHERE pdb='{1}' AND chain='{2}');\n".format(tables['chain'], self.id, h.chain)
                dataset = 0
                if int(h.pos) <= self.get_chain_by_id(h.chain).last_structure.number: dataset = 1
                command += "INSERT INTO {0}(chain,position,hetero,inchain) VALUES (@chain, {1.pos}, '{1.id}', {2});\n".format(tables['hetchn'], h, dataset)
        for m in pdbheader.molecules:
            command += "INSERT INTO {0} VALUES ();\n".format(tables['repeidx'])
            for c in pdbheader.molecules[m].chains:
                if c in self._chain_id:  #sometimes a chain is described in the header that has no relation in the coordinates
                    command += "SET @chain = (SELECT nid FROM {0} WHERE pdb='{1}' AND chain='{2}');\n".format(tables['chain'], self.id, c)
                    command += "INSERT INTO {0} VALUES (LAST_INSERT_ID(), @chain);\n".format(tables['repe'])
        for s in pdbheader.sites:
            if pdbheader.sites[s].bind is not None:
                bdata = pdbheader.sites[s].bind
                if not isinstance(bdata[1], list):
                    bdata[1] = bdata[1].split()
                    if len(bdata[1]) == 1:
                        bdata[1].append(bdata[1][0][1:].strip())
                        bdata[1][0] = bdata[1][0][0]
                if (len(bdata[1]) == 0): #only one het of this type in the pdb
                    for e in pdbheader.hetero:
                        if bdata[0] == e.id:
                            bdata[1].append(e.chain)
                            bdata[1].append(e.pos)
                import re
                posidx = re.compile('(\-*\d+)(\w*)')
                m      = posidx.match(bdata[1][1])
                pos    = m.group(1)
                idx    = m.group(2)
                command += "SET @chain = (SELECT nid FROM {0} WHERE pdb='{1}' AND chain='{2[1][0]}');\n".format(tables['chain'], self.id, bdata)
                command += "SET @bind = (SELECT nid FROM {0} WHERE chain=@chain AND position={3} AND hetero='{2[0]}');\n".format(tables['hetchn'], self.id, bdata, pos)
                command += "INSERT INTO {0}(pdb,name,description,bind) VALUES ('{1}','{2.id}','{2.desc}', @bind);\n".format(tables['site'],self.id,pdbheader.sites[s])
            else:
                command += "INSERT INTO {0}(pdb,name,description) VALUES ('{1}','{2.id}','{2.desc}');\n".format(tables['site'],self.id,pdbheader.sites[s])
            for p in pdbheader.sites[s].spec:
                command += "SET @chain = (SELECT nid FROM {0} WHERE pdb='{1}' AND chain='{2}');\n".format(tables['chain'], self.id, p[1])
                idx = ' '
                num = p[2]
                try:
                    int(num[-1])
                    idx = ' '
                    num = p[2]
                except:
                    idx = num[-1]
                    num = num[:-1]
                command += "INSERT IGNORE INTO {0} VALUES(LAST_INSERT_ID(),@chain,{1},'{2}','{3}');\n".format(tables['sitepos'],num, idx, p[0])

        command += self.innercontacts.toSQL()
        command += self.interfaces.toSQL()

        return command



from . import process_COMPND_line
from . import process_SOURCE_line
from . import fix_chain

from SBI.beans.JSONer import JSONer
import re


class Molecule(JSONer):

    '''
    Includes the data from COMPND and SOURCE.
    Each one includes several tokens; those of COMPND:

    TOKEN           VALUE DEFINITION
    -------------------------------------------------------------------------
    MOL_ID          Numbers each component; also used in  SOURCE to associate
                    the information.
    MOLECULE        Name of the macromolecule.
    CHAIN           Comma-separated list of chain  identifier(s).
    FRAGMENT        Specifies a domain or region of the  molecule.
    SYNONYM         Comma-separated list of synonyms for  the MOLECULE.
    EC              The Enzyme Commission number associated  with the molecule.
                    If there is more than one EC number,  they are presented
                    as a comma-separated list.
    ENGINEERED      Indicates that the molecule was  produced using
                    recombinant technology or by purely  chemical synthesis.
    MUTATION        Indicates if there is a mutation.
    OTHER_DETAILS   Additional comments.

    And those of SOURCE:

    TOKEN                     VALUE  DEFINITION
    --------------------------------------------------------------------------
    MOL_ID                    Numbers each molecule. Same as appears in COMPND
    SYNTHETIC                 Indicates a  chemically-synthesized source.
    FRAGMENT                  A domain or  fragment of the molecule may be
                              specified.
    ORGANISM_SCIENTIFIC       Scientific name of the  organism.
    ORGANISM_COMMON           Common name of the  organism.
    ORGANISM_TAXID            NCBI Taxonomy ID number  of the organism.
    STRAIN                    Identifies the  strain.
    VARIANT                   Identifies the  variant.
    CELL_LINE                 The specific line of cells used in
                              the experiment
    ATCC                      American Type  Culture Collection tissue
                              culture  number.
    ORGAN                     Organized group of  tissues that carries on
                              a specialized function.
    TISSUE                    Organized group  of cells with a common
                              function and  structure.
    CELL                      Identifies the  particular cell type.
    ORGANELLE                 Organized structure  within a cell.
    SECRETION                 Identifies the secretion, such as  saliva, urine,
                              or venom,  from which the molecule was isolated.
    CELLULAR_LOCATION         Identifies the location  inside/outside the cell.
    PLASMID                   Identifies the plasmid  containing the gene.
    GENE                      Identifies the  gene.
    EXPRESSION_SYSTEM         Scientific name of the organism in  which the
                              molecule was expressed.
    EXPRESSION_SYSTEM_COMMON  Common name of the organism in which the
                              molecule was  expressed.
    EXPRESSION_SYSTEM_TAXID   NCBI Taxonomy ID of the organism  used as the
                              expression  system.
    EXPRESSION_SYSTEM_STRAIN  Strain of the organism in which  the molecule
                              was  expressed.
    EXPRESSION_SYSTEM_VARIANT Variant of the organism used as the
                              expression  system.
    EXPRESSION_SYSTEM_CELL_LINE  The specific line of cells used as  the
                                 expression  system.
    EXPRESSION_SYSTEM_ATCC_NUMBER  Identifies the ATCC number of the
                                   expression system.
    EXPRESSION_SYSTEM_ORGAN      Specific organ which expressed  the molecule.
    EXPRESSION_SYSTEM_TISSUE     Specific tissue which expressed  the molecule.
    EXPRESSION_SYSTEM_CELL       Specific cell type which  expressed the
                                 molecule.
    EXPRESSION_SYSTEM_ORGANELLE  Specific organelle which expressed
                                 the molecule.
    EXPRESSION_SYSTEM_CELLULAR_LOCATION  Identifies the location inside or
                                         outside the cell  which expressed
                                         the molecule.
    EXPRESSION_SYSTEM_VECTOR_TYPE   Identifies the type of vector used,  i.e.,
                                    plasmid,  virus, or cosmid.
    EXPRESSION_SYSTEM_VECTOR      Identifies the vector used.
    EXPRESSION_SYSTEM_PLASMID     Plasmid used in the recombinant experiment.
    EXPRESSION_SYSTEM_GENE        Name of the gene used in  recombinant
                                  experiment.
    OTHER_DETAILS                 Used to present  information on the
                                  source which
                                  is not  given elsewhere.
    '''
    def __init__(self, pdb):
        self._pdb      = pdb
        self._COMPND   = ''
        self._SOURCE   = ''
        self._chains   = []
        self._name     = ''
        self._ec       = set()
        self._taxid    = set()
        self._processd = False

    #
    # ATTRIBUTES
    #
    @property
    def pdb(self):
        return self._pdb

    @property
    def chains(self):
        return self._chains

    @property
    def name(self):
        return self._name

    @property
    def ec(self):
        return self._ec

    @property
    def taxid(self):
        return self._taxid

    #
    # BOOLEANS
    #
    @property
    def is_processed(self):
        return self._processd

    #
    # READ FUNCTIONS
    #
    def add_line(self, switch, line):
        if switch == 'COMPND':
            self._COMPND += process_COMPND_line(line)
        elif switch == 'SOURCE':
            self._SOURCE += process_SOURCE_line(line)

    #
    # PIVATE FUNCTIONS
    #
    def _parse(self):
        for field in self._SOURCE.split(';'):
            if field.startswith('ORGANISM_TAXID'):
                for tax in re.split(',', field.split(':')[1].strip()):
                    try:  # avoid add tag 'CHEMICALLY SYNTHESIZED' #4F4Z
                        int(tax.strip())
                        self.taxid.add(tax.strip())
                    except:
                        pass
                continue
            if field.strip() == 'SYNTHETIC: YES':
                if len(self.taxid) == 0:
                    self.taxid.add('32630')  # synthetic construct
                continue
        self._processd = True

    def _parse_cmpnd(self):
        # [EXCEPTION]. Found, at least, in pdb: 1TCR
        self._COMPND = re.sub('\\\;', ',', self._COMPND)

        for field in self._COMPND.split(';'):
            fdata = field.split(':')
            if field.startswith('CHAIN:'):
                self._chains = [fix_chain(f.strip()) for f in fdata[1].strip().split(',')]
                # [EXCEPTION]. Found in 2C14 and others
                if '' in self._chains:
                    self._chains.remove('')
                continue
            if field.startswith('MOLECULE:'):
                self._name = ":".join(fdata[1:]).strip()
                continue
            if field.startswith('EC:'):
                self._ec = set([x.strip() for x in fdata[1].split(',')])
                continue

    #
    # OUT FUNCTIONS
    #
    def as_dict(self):
        nobj = {}
        nobj['name']   = self.name
        nobj['chains'] = self.chains
        nobj['ec']     = list(self.ec)
        nobj['taxid']  = list(self.taxid)
        return nobj

    def __repr__(self):
        return self.json()



from .contact      import Contact, ContactAA, ContactAN, ContactAH

from .interface    import Interface, PPInterface, PNInterface, PHInterface

from .inner        import PPInnerContact, PHInnerContact

from Complex       import Complex
from InnerContacts import InnerContacts


#from FTDock import *
from HEXDock import *
from ZDock import *
from PATCHDock import *
from SplitPotentials import SplitPotentialsMatrix
from SplitPotentialsPPI import *
from ScoringFunctions import *



import sys
import os
import argparse
import gzip

from SBI.beans     import Path
from SBI.data      import element_dic
from SBI.databases import PDBeChemlink

from archdb.src import Source, PDBeChem, Element
from archdb.src import start_transaction, end_transaction


# User Options
def options(*args, **kwds):
    parser = argparse.ArgumentParser(prog = 'PDBeChem2SQL', formatter_class = argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-d', '--database', dest     = "database",
                        action = "store", required = True, help = "Destination Directory for PDBeChem database.")
    parser.add_argument('-s', '--sql', dest     = "sqlfile",
                        action = "store", required = True, help = "Output SQL file")
    parser.add_argument('-k', '--skipdownload', dest     = "skip_download",
                        action = "store_true", default  = False, help = "Skips downloading the database.")
    parser.add_argument('-v', '--verbose', dest     = "verbose",
                        action = "store_true", default  = False, help = "Verbose Mode.")
    options = parser.parse_args()
    return options


def PDBeChem2SQL(database, sqlfile, skip_download, verbose):
    pdbechem_connect = PDBeChemlink(local = database)
    newsource        = None
    if not skip_download:
        if verbose: sys.stderr.write("Downloading PDBeChem database to {0} ...\n".format(database))
        pdbechem_connect.download()
        newsource = Source(name   = 'PDBeChem', source = pdbechem_connect.source)
        if verbose: sys.stderr.write("Download Finished.\n")
    else:
        if verbose: sys.stderr.write("Using previously downloaded database.\n")

    noparent_chems = []
    parent_chems   = []
    if verbose: sys.stderr.write("Parsing PDBeChem.\n")
    for chem_file in pdbechem_connect.localPDBeChems:
        if verbose: sys.stderr.write("\tReading {0} ....\n".format(chem_file))
        newchem = PDBeChem(chem_file)
        if newchem.parent is None: noparent_chems.append(newchem.toSQL())
        else: parent_chems.append(newchem.toSQL())

    if verbose: sys.stderr.write("Writing {0} ....\n".format(sqlfile))
    Path.mkdir(os.path.split(os.path.abspath(sqlfile))[0])
    sql_fd = gzip.open(sqlfile, 'wb')
    sql_fd.write(start_transaction())
    if newsource is not None:
        sql_fd.write(newsource.toSQL())
    for e in element_dic.values():
        newelement = Element(e.number, e.symbol, e.name)
        sql_fd.write(newelement.toSQL() + "\n")
    sql_fd.write("\n".join(noparent_chems) + "\n")
    sql_fd.write("\n".join(parent_chems) + "\n")
    sql_fd.write(end_transaction())
    sql_fd.close()
    if verbose: sys.stderr.write("End execution.\n")

if __name__ == "__main__":
    options = options()
    PDBeChem2SQL(options.database, options.sqlfile, options.skip_download, options.verbose)



# -*-
#
# @author: jaumebonet
# @email:  jaume.bonet@gmail.com
# @url:    jaumebonet.github.io
#
# @date:   2015-05-08 16:15:32
# @lab:    LPDI/EPFL
#
# @last modified by:   jaumebonet
# @last modified time: 2015-05-10 15:07:37
#
# -*-
import re

from argparse import ArgumentParser
from argparse import ArgumentDefaultsHelpFormatter


def basic_parser(*args, **kargs):
    '''
    Include those options that most likely are going to be shared
    by all (or most of) the modpy scripts.

    Advantages:
        1) Write less
        2) Homogenization of parameters

    @return: argparse.ArgumentParser object
    '''
    parser = ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter)

    parser.add_argument('--pir', dest='alignment', type=str, action='store',
                        help="PIR formated alignment", metavar="PIR_FILE")
    parser.add_argument("--out", dest="out", type=str, action="store",
                        default=None, metavar="OUT_PREFIX",
                        help="Prefix for the log and error file")
    parser.add_argument("-v", dest="verbose", action="store_true",
                        default=False, help="Verbose Mode")
    return parser


def identify_pir(filename):
    '''
    Identify known structures and query sequence in the PIR alignment.

    @return: {'str': [str(), ...], 'seq': str()}
    '''
    _idr = re.compile('^>\w+\;(\S+)')
    data = {'str': [], 'seq': None}

    with open(filename) as fd:
        nextline = False
        mcontent = None
        for line in fd:
            m = _idr.match(line)
            if m:
                nextline = True
                mcontent = m.group(1).strip()
                continue
            if nextline and line.startswith('structureX'):
                    data['str'].append(mcontent)
                    nextline, mcontent = False, None
            elif nextline and line.startswith('sequence'):
                    data['seq'] = mcontent
                    nextline, mcontent = False, None
    return data



"""
jbonet @ boliva's lab 2013
"""
from SBI.beans import StorableObject
from SBI.beans import JSONer
from SBI.beans import File

import numpy as np
from collections import Counter


class Arch(StorableObject, JSONer):

    def __init__(self, source, ss1, ss2, internalss, distanceAA, structure,
                 torsionsCA, PsiPhi, order):
        self._source = source
        self._ss1    = ss1  # N-term
        self._ss2    = ss2  # C-term
        self._intss  = internalss  # number of internal secondary structures
        self._inttxt = []          # header format of internal ss (if any)
        self._inttyp = ''          # internal structures type (if any)
        self._distAA = distanceAA
        self._str    = structure
        self._vlo    = self._ss2._tsf44 - self._ss1._tsf11
        self._dist   = np.sqrt(np.dot(self._vlo, self._vlo))
        self._angls  = [None, None, None]  # theta, rho, delta
        self._torCA  = torsionsCA
        self._psiphi = PsiPhi
        self._order  = order
        self._tordis = self._distance_torsion_vector()
        self._angles()
        # theta, rho, delta
        self._torgls = [self._angle_torsion_vector(self._angls[0], 180),
                        self._angle_torsion_vector(self._angls[1], 360),
                        self._angle_torsion_vector(self._angls[2], 180)]
        self._htotal = 0
        self._hss    = 0
        self._hloop  = 0
        self._hdim   = 0
        self._hmxini = 0
        self._hmatix = self._calculate_hydrogen_bond_matrix()

        distribution = Counter(list(self.structure_sequence))
        self._gaps   = distribution['x'] if 'x' in distribution else 0

    @property
    def type(self):
        if self._ss1._sstype == 'E' and self._ss2._sstype == 'E':
            if self._hss < 1:
                # If this condition was == 0 some BK of length 1 can become BN
                return 'BK'
            else:
                if self.aminoacid_distance == 1 and self.theta < 160:
                    return 'BK'
                if self.theta < 90:
                    return 'BK'
                return 'BN'
        else:
            return self._ss1._sstype + self._ss2._sstype

    @property
    def boundaries(self):
        return self.initial_position + '-' + self.end_position

    @property
    def initial_position(self):
        return str(self._ss1._inip).strip()

    @property
    def end_position(self):
        return str(self._ss2._endp).strip()

    @property
    def selfboundaries(self):
        return '0-' + str(len(self._torCA) - 1)

    @property
    def access_surface(self):
        sec = []
        for residues in self._str.aminoacids:
            sec.append(residues.accessibilitycoded)
        return ''.join(sec)

    @property
    def psiphi(self):
        data = ['', '', '']
        for i in range(0, len(self._psiphi)-4, 3):
            data[0] += self._psiphi[i]
            data[1] += self._psiphi[i+1]
            data[2] += self._psiphi[i+2]
        return data

    @property
    def cartesian_distance(self):
        return self._dist

    @property
    def aminoacid_distance(self):
        return self._distAA

    @property
    def length(self):
        return self._distAA

    @property
    def internal_structures(self):
        return self._intss

    @property
    def gaps(self):
        return self._gaps

    @property
    def identifier(self):
        ident = [self._source, str(self._ss1._inip).strip()]
        if self.is_superarch:
            ident.append(str(self._intss))
        return "_".join(ident)

    @property
    def theta(self):
        return self._angls[0]

    @property
    def rho(self):
        return self._angls[1]

    @property
    def delta(self):
        return self._angls[2]

    @property
    def aminoacid_sequence(self):
        return self._str.gapped_protein_sequence

    @property
    def structure_sequence(self):
        return self._str.gapped_protein_secondary_structure

    @property
    def structure(self):
        return self._str

    @property
    def length_ss1(self):
        return self._ss1._length

    @property
    def length_ss2(self):
        return self._ss2._length

    @property
    def has_gaps(self):
        return self._gaps > 0

    @property
    def is_superarch(self):
        return self._intss > 0

    def unbound_ss(self, percentage):
        if not self.has_gaps:
            return False
        return (self._gaps/self._distAA)*100 >= percentage

    def _calculate_hydrogen_bond_matrix(self):
        matrix_min_border_index = self._str._get_structure_array_coordinate(self._ss1._endp)
        matrix_max_border_index = self._str._get_structure_array_coordinate(self._ss2._inip)
        ss1_end_index           = self._str._get_structure_array_coordinate(self._ss1._endp)
        ss2_ini_index           = self._str._get_structure_array_coordinate(self._ss2._inip)

        if matrix_min_border_index > 1:
            matrix_min_border_index -= 2
        else:
            matrix_min_border_index  = 0
        if matrix_max_border_index < len(self._str.aminoacids) - 2:
            matrix_max_border_index += 2
        else:
            matrix_max_border_index = len(self._str.aminoacids) - 1

        matrix_size = matrix_max_border_index - matrix_min_border_index + 1
        matrix = np.zeros(shape=(matrix_size, matrix_size))
        ii = 0
        for i in range(matrix_min_border_index, matrix_min_border_index + matrix_size):
            jj = 0
            for j in range(matrix_min_border_index, matrix_min_border_index + matrix_size):
                diff = j-i
                if self._str.aminoacids[i].dssp.nhoa[0] != diff and self._str.aminoacids[i].dssp.nhob[0] != diff:
                    matrix[ii][jj] = 0
                elif self._str.aminoacids[i].dssp.nhoa[0] == diff and self._str.aminoacids[i].dssp.nhob[0] == diff:
                    matrix[ii][jj]  = self._str.aminoacids[i].dssp.nhoa[1]
                    matrix[ii][jj] += self._str.aminoacids[i].dssp.nhob[1]
                elif self._str.aminoacids[i].dssp.nhoa[0] == diff and self._str.aminoacids[i].dssp.nhob[0] != diff:
                    matrix[ii][jj]  = self._str.aminoacids[i].dssp.nhoa[1]
                else:
                    matrix[ii][jj]  = self._str.aminoacids[i].dssp.nhob[1]
                jj += 1
            ii += 1
        jj = 0
        for j in range(matrix_min_border_index, matrix_min_border_index + matrix_size):
            ii = 0
            for i in range(matrix_min_border_index, matrix_min_border_index + matrix_size):
                diff = i-j
                if self._str.aminoacids[j].dssp.ohna[0] != diff and self._str.aminoacids[j].dssp.ohnb[0] != diff:
                    matrix[ii][jj] += 0
                elif self._str.aminoacids[j].dssp.ohna[0] == diff and self._str.aminoacids[j].dssp.ohnb[0] == diff:
                    matrix[ii][jj] += self._str.aminoacids[j].dssp.ohna[1]
                    matrix[ii][jj] += self._str.aminoacids[j].dssp.ohnb[1]
                    matrix[ii][jj]  = matrix[ii][jj]/2
                elif self._str.aminoacids[j].dssp.ohna[0] == diff and self._str.aminoacids[j].dssp.ohnb[0] != diff:
                    matrix[ii][jj] += self._str.aminoacids[j].dssp.ohna[1]
                    matrix[ii][jj]  = matrix[ii][jj]/2
                else:
                    matrix[ii][jj] += self._str.aminoacids[j].dssp.ohnb[1]
                    matrix[ii][jj]  = matrix[ii][jj]/2
                ii += 1
            jj += 1
        hbond_total, hbond_ss, hbond_loop = 0, 0, 0
        for j in range(matrix_size):
            for i in range(matrix_size):
                if matrix[i][j] <= -0.01:
                    hbond_total += 1
                if matrix[i][j] <= -0.6:
                    position_i = matrix_min_border_index + i
                    position_j = matrix_min_border_index + j
                    if (position_i <= ss1_end_index and position_j >= ss2_ini_index) or \
                       (position_i >= ss2_ini_index and position_j <= ss1_end_index):
                        hbond_ss += 1
                    elif (position_i > ss1_end_index and position_i < ss2_ini_index) or \
                         (position_j > ss1_end_index and position_j < ss2_ini_index):
                            hbond_loop += 1

        self._htotal = hbond_total
        self._hss    = hbond_ss
        self._hloop  = hbond_loop
        self._hdim   = matrix_size
        self._hmxini = matrix_min_border_index

        return matrix

    def _distance_torsion_vector(self):

        data    = ["-" for i in range(len(self._torCA))]
        border1 = self._ss1._length - 1
        border2 = border1 + self._distAA + 1
        if self._dist   <= 7.5:
            data[border1], data[border2] = 's', 's'
        elif self._dist <= 10:
            data[border1], data[border2] = 'm', 'm'
        elif self._dist <= 20:
            data[border1], data[border2] = 'l', 'l'
        else:
            data[border1], data[border2] = 'L', 'L'
        return ''.join(data)

    def _angle_torsion_vector(self, angle, max_range):
        data     = ["-" for i in range(len(self._torCA))]
        border1  = self._ss1._length - 1
        border2  = border1 + self._distAA + 1
        accuracy = 36
        definitions = 'ABCDEFGHIJKLMNOPQRSTUVXYZabcdefghijk'
        for i in range(accuracy):
            if angle >= i * (max_range/accuracy) and angle < (i+1) * (max_range/accuracy):
                break
        data[border1] = definitions[i]
        data[border2] = definitions[i]
        return ''.join(data)

    def _angles(self):

        normal = self._vectorproduct(self._vlo, self._ss1._eigf11)
        target = self._vectorproduct(self._ss1._eigf11, normal)
        const  = np.dot(target, target)

        if const != 0:
            self._angls[0] = np.degrees(np.arccos(np.dot(self._ss1._eigf11, self._ss2._eigf44)))

            const2 = np.dot(self._ss2._eigf44, self._ss1._eigf11)
            proj   = np.subtract(self._ss2._eigf44, np.multiply(const2, self._ss1._eigf11))
            dproj  = np.sqrt(np.dot(proj, proj))

            if dproj != 0:
                angle  = np.degrees(np.arccos(np.dot(proj, normal)/dproj))
                const3 = np.dot(proj, target)
                if const3 < 0:
                    angle = 360 - angle
                self._angls[1] = angle
            else:
                self._angls[1] = 0

        else:
            angle = np.degrees(np.arccos(np.dot(self._ss1._eigf11, self._ss2._eigf44)))
            self._angls[0] = angle
            self._angls[1] = 500

        self._angls[2] = np.degrees(np.arccos(np.dot(self._ss1._eigf11,self._vlo)/self._dist))

    def _vectorproduct(self, v, n):

        retvec    = [0, 0, 0]
        retvec[0] = v[1]*n[2] - v[2]*n[1]
        retvec[1] = v[2]*n[0] - v[0]*n[2]
        retvec[2] = v[0]*n[1] - v[1]*n[0]

        return np.divide(retvec, np.sqrt(np.dot(retvec,retvec)))

    def _format_hbond_matrix(self):
        data = []
        data.append('______|')
        data.append('      |')
        ii = 0
        for i in range(self._hmxini, self._hmxini + self._hdim):
            data[0] = data[0] + '{0:>5d}|'.format(i)
            data[1] = data[1] + '-----+'
            data.append('{0:>4d}  |'.format(i))
            data.append('      |')
            jj = 0
            for j in range(self._hmxini, self._hmxini + self._hdim):
                data[-2] = data[-2] + '{0:>5.1f}|'.format(self._hmatix[ii][jj])
                data[-1] = data[-1] + '-----+'
                jj += 1
            ii += 1
        return "\n".join(data)

    def _format_border_hbond_matrix(self):
        data = []
        data.append('______|')
        data.append('      |')
        ii = 0
        for i in range(self._hmxini, self._hmxini + self._hdim):
            if i < self._ss1._length or i >= self._ss1._length + self._distAA:
                data[0] = data[0] + '{0:>5d}|'.format(i)
                data[1] = data[1] + '-----+'
                data.append('{0:>4d}  |'.format(i))
                data.append('      |')
                jj = 0
                for j in range(self._hmxini, self._hmxini + self._hdim):
                    if j < self._ss1._length or j >= self._ss1._length + self._distAA:
                        data[-2] = data[-2] + '{0:>5.1f}|'.format(self._hmatix[ii][jj])
                        data[-1] = data[-1] + '-----+'
                    jj += 1
            ii += 1
        return "\n".join(data)

    def format2file(self, filename, extension = 'pdb', center = False):
        if extension not in ('pdb', 'js'):
            raise AttributeError('Not accepted extension')
        structure = File('.'.join([filename, extension]), 'w')
        if extension == 'pdb':
            structure.write(self.pdb_format(center = center))
        elif extension == 'js':
            structure.write(self.js_format(center = center))
        structure.close()

    def pdb_format(self, center = False):
        data = []
        data.append(self._ss1.headerformat())
        for internalss in self._inttxt:
            data.append(internalss)
        data.append(self._ss2.headerformat(self._intss + 2, chr(65 + self._intss + 1)))
        tempstr = self._str.duplicate(hetero = False, water = False)
        if center:
            tempstr.translate_onto_origin()
        data.append(tempstr.PDB_format())
        return "\n".join(data)

    def js_format(self, center = False):
        init = 'var {0}="'.format('pdb_' + self.identifier)
        return init + self.pdb_format(center = center).replace('\n', '\\n') + '"'

    def as_dict(self):
        return {'id'         : self.identifier,
                'type'       : self.type,
                'aadistance' : self.aminoacid_distance,
                'distance'   : self.cartesian_distance,
                'inipos'     : self.initial_position,
                'endpos'     : self.end_position,
                'theta'      : self.theta,
                'rho'        : self.rho,
                'delta'      : self.delta,
                'gaps'       : self.gaps,
                'ss1L'       : self.length_ss1,
                'ss2L'       : self.length_ss2,
                'sequence'   : self.aminoacid_sequence,
                'structure'  : self.structure_sequence}

    def json_format(self, as_string=True):
        return self.json() if as_string else self.as_dict()

    def archtype_header(self):
        data = []
        data.append('CODE: {0._source}'.format(self))
        data.append('CHAIN: {0}'.format(self._source.split('_')[1]))
        data.append('TYPE: {0.type}'.format(self))
        data.append('ORDER: {0._order}'.format(self))
        data.append('ID_AA: {0.selfboundaries}'.format(self))
        data.append('REAL_AA: {0.boundaries}'.format(self))
        if self.is_superarch:
            data.append('INT_SS: {0._intss}'.format(self))
            data.append('INT_SS_TYPE: {0._inttyp}'.format(self))
        return "\n".join(data)

    def archtype_loop(self):
        loopinfo = []
        loopinfo.append('LOOP:')
        loopinfo.append('{0._distAA:>3d}'.format(self))
        loopinfo.append('{0._ss1._length:>3d}'.format(self))
        loopinfo.append('{0._ss2._length:>3d}'.format(self))
        loopinfo.append('{0:>3d}'.format(self._ss1.get_moment_of_inertia_length('f11')))
        loopinfo.append('{0:>3d}'.format(self._ss2.get_moment_of_inertia_length('f44')))
        loopinfo.append('{0:>.6f}'.format(self._dist))
        loopinfo.append('{0:>.6f}'.format(self._angls[0]))
        loopinfo.append('{0:>.6f}'.format(self._angls[1]))
        loopinfo.append('{0:>.6f}'.format(self._angls[2]))
        return '\t'.join(loopinfo)

    def archtype_hbond(self, matrix = True):
        data = []
        data.append('HBOND_DIM:  {0:>3d}'.format(self._hdim))
        data.append('HBOND_LOOP: {0:>3d}'.format(self._distAA))
        data.append('HBOND_NUM:  \t{0:>3d}\t{1:>3d}\t{2:>3d}\n'.format(self._htotal, self._hss, self._hloop))
        if matrix:
            data.append('HBOND_MATRIX\n\n' + self._format_hbond_matrix())
        return "\n".join(data)

    def archtype_psiphi(self):
        data = []
        psiphi = self.psiphi
        data.append('PSI_PHIA: {0}'.format(psiphi[0]))
        data.append('PSI_PHIB: {0}'.format(psiphi[1]))
        data.append('PSI_PHIC: {0}'.format(psiphi[2]))
        return "\n".join(data)

    def archtype_format(self, matrix = True):
        data = []
        data.append('\n******\n')
        data.append(self.archtype_header())
        data.append(self.archtype_loop())
        data.append('\nSEQUENCE: {0}'.format(self.aminoacid_sequence))
        data.append('TOR_CA  : {0}'.format(self._torCA))
        data.append('ACCES_SU: {0}'.format(self.access_surface))
        data.append(self.archtype_psiphi())
        data.append('SEC_STR : {0}'.format(self.structure_sequence))
        data.append('TOR_THET: {0}'.format(self._torgls[0]))
        data.append('TOR_RHO : {0}'.format(self._torgls[1]))
        data.append('ANG_DELT: {0}'.format(self._torgls[2]))
        data.append('DISTANCE: {0}\n'.format(self._tordis))
        data.append(self.archtype_hbond(matrix))
        return "\n".join(data)

    def line_format(self, sequence = False):
        data = [self.identifier, self.type, str(self.aminoacid_distance)]
        data.append('{0:07.3f}'.format(self.cartesian_distance))
        data.append('{0:07.3f}'.format(self.theta))
        data.append('{0:07.3f}'.format(self.rho))
        data.append('{0:07.3f}'.format(self.delta))
        data.append(str(self.gaps))
        if sequence:
            data.extend([self.aminoacid_sequence, self.structure_sequence])
        return "\t".join(data)

    def __lt__(self, other):
        return self._distAA < other._distAA

    def __le__(self, other):
        return self.__lt__(other) or self.__eq__(other)

    def __eq__(self, other):
        return self._distAA == other._distAA

    def __ne__(self, other):
        return not self.__eq__(other)

    def __gt__(self, other):
        return self._distAA > other._distAA

    def __ge__(self, other):
        return self.__gt__(other) or self.__eq__(other)

    def __repr__(self):
        data = []
        data.append('**Secondary Structure Relation:')
        data.append('STR1:\t{0}'.format(self._ss1.strdata('f11')))
        data.append('STR2:\t{0}'.format(self._ss2.strdata('f44')))
        data.append('ARCH TYPE: {0.type}'.format(self))
        data.append('INTERNAL SS: {0._intss}'.format(self))
        data.append('DISTANCE: {0._dist}'.format(self))
        data.append('THETA: {0.theta:<.5f} RHO: {0.rho:<.5f} DELTA: {0.delta:<.5f}'.format(self))
        data.append('**\n\n')
        return "\n".join(data)



from SBI.databases.uniprot import Uniprot

tables = {'uniprot' : 'uniprot',
          'ent2acc' : 'uniprot_entry2accession',
          'uni2go'  : 'uniprot2GO',
          'uni2tax' : 'uniprot2taxid',
          'unienz'  : 'uniprot2enzyme',

          'taxid'   : 'taxid',
          'taxold'  : 'taxid_old',

          'go'      : 'GO',
          'GO_alt'  : 'GO_alternative'}

class Uniprot(Uniprot):
    def __init__(self, entry = None, source = None, inline = None):
        super(Uniprot, self).__init__(entry, source, inline)

    def toSQL(self):
        command = []
        command.append('INSERT INTO {0} VALUES ("{1.entry}","{1.source}");\n'.format(tables['uniprot'],self))
        for a in self.accession:
            command.append('INSERT INTO {0} VALUES ("{1.entry}","{2}");\n'.format(tables['ent2acc'], self, a))
        if self.taxid is not None:
            command.append('SET @taxid = (SELECT COALESCE((SELECT id FROM {0} WHERE id={1}),(SELECT taxid FROM {2} WHERE oldid={1})));\n'.format(tables['taxid'], self.taxid, tables['taxold']))
            command.append('INSERT INTO {0} VALUES ("{1.entry}",@taxid,"is");\n'.format(tables['uni2tax'], self))
        for h in self.hosts:
            command.append('SET @taxid = (SELECT COALESCE((SELECT id FROM {0} WHERE id={1}),(SELECT taxid FROM {2} WHERE oldid={1})));\n'.format(tables['taxid'], h, tables['taxold']))
            command.append('INSERT INTO {0} VALUES ("{1.entry}",@taxid,"host");\n'.format(tables['uni2tax'], self))
        for db in self.databases:
            if db[0] == 'GO':
                command.append('SET @goid = (SELECT COALESCE((SELECT nid FROM {0} WHERE id="{1}"),(SELECT GO_nid FROM {2} WHERE alternative="{1}")));\n'.format(tables['go'], db[1], tables['GO_alt']))
                command.append('INSERT IGNORE INTO {0} VALUES ("{1}",@goid);\n'.format(tables['uni2go'], self.entry))
        return "".join(command)



import sys, os, argparse, gzip

from SBI               import SBIglobals
from SBI.beans           import Path
from SBI.databases import PDBlink

from archdb.src import Source, PDB
from archdb.src import start_transaction, end_transaction

# User Options
def options(*args, **kwds):
    parser = argparse.ArgumentParser(prog = 'PDB2SQL',  formatter_class = argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-d', '--database',     dest     = "database",
                        action = "store",       required = True,  help = "Destination Directory for PDB database.")
    parser.add_argument('-q', '--seqdatabase',  dest     = "seqdatabase",
                        action = "store",       required = True,  help = "Destination Directory for PDBseq database.")
    parser.add_argument('-l', '--listfiles',    dest     = "listfiles",
                        action = "store",       default  = None,  help = "In case one needs to run only a subset.")
    parser.add_argument('-s', '--sql',          dest     = "sqlfile",
                        action = "store",       required = True,  help = "Output SQL dir")
    parser.add_argument('-k', '--skipdownload', dest     = "skip_download",
                        action = "store_true",  default  = False,  help = "Skips downloading the database.")
    parser.add_argument('-v', '--verbose',      dest     = "verbose",
                        action = "store_true",  default  = False, help = "Verbose Mode.")
    options = parser.parse_args()
    return options

def PDB2SQL(database, seqdatabase, listfiles, sqlfile, skip_download, verbose):
    pdb_connect = PDBlink(local  = database, PDBseq = seqdatabase)
    newsource   = None

    if not skip_download:
        if verbose: sys.stderr.write("Syncronizing PDB database to {0} ...\n".format(database))
        pdb_connect.sync_PDB(log_file=os.path.join(database,'PDB.sync.log'))
        newsource = Source(name = 'PDB', source = pdb_connect.source)
        if verbose: sys.stderr.write("Creating PDBseq in {0} ...\n".format(seqdatabase))
        pdb_connect.make_PDBseq(log_file = os.path.join(seqdatabase,'PDB.seq.log'))
        if verbose: sys.stderr.write("Download Finished.\n")
        outdir = os.path.abspath(os.path.join(sqlfile,'00'))
        Path.mkdir(outdir)
        sql_fd = gzip.open(os.path.join(outdir, '0000.sql.gz'), 'wb')
        sql_fd.write(start_transaction())
        sql_fd.write(newsource.toSQL())
        sql_fd.write(end_transaction())
        sql_fd.close()
    else:
        if verbose: sys.stderr.write("Using previously downloaded database.\n")

    files2check = set()
    if listfiles is not None:
        fd = open(listfiles)
        for line in fd:
            files2check.add(line.strip())
        fd.close()
        logfd = open(listfiles + ".log","w")
    else:
        logfd = open("PDB2SQL.log","w")
    import traceback
    for pdbfile in pdb_connect.localPDBs:
        try:
            if listfiles is not None and pdbfile not in files2check:
                if len(files2check) == 0: break
                continue
            #else:
               # files2check.add(pdbfile)
               # files2check.remove(pdbfile)
            if verbose: sys.stderr.write("Working file {0}\n".format(pdbfile))
            newPDB = PDB(pdb_file = pdbfile)
            outsqldir  = os.path.join(sqlfile,newPDB.id[1:3].lower())
            Path.mkdir(outsqldir)
            outsqlfile = os.path.join(outsqldir, newPDB.id + '.sql.gz')
            # outsqlfile = os.path.join(os.getcwd(), newPDB.id + '.sql.gz')
            if verbose: sys.stderr.write("\tOutput SQL file is {0}.\n".format(outsqlfile))
            sql_fd = gzip.open(outsqlfile,'wb')
            sql_fd.write(start_transaction())
            sql_fd.write(PDB.preuniprotdeleted())
            sql_fd.write(newPDB.toSQL())
            sql_fd.write(PDB.afteruniprotdeleted())
            sql_fd.write(end_transaction())
            sql_fd.close()
        except KeyboardInterrupt:
            raise
        except:
            if verbose: sys.stderr.write("\tAn error occurred. Check log file\n")
            SBIglobals.alert('error', None, '\tAn error occurred for {0} . Check log file'.format(pdbfile))
            logfd.write("FILE {0}\n".format(pdbfile))
            logfd.write(traceback.format_exc())
            logfd.write("\n");

if __name__ == "__main__":
    SBIglobals.deepdebug = False
    options = options()
    if options.listfiles is not None:
        SBIglobals.output_file(options.listfiles + ".debug")
    PDB2SQL(options.database, options.seqdatabase, options.listfiles, options.sqlfile, options.skip_download, options.verbose)



import sys, os, argparse, gzip

from SBI.beans           import Path

from archdb.src import CDhit
from archdb.src import start_transaction, end_transaction

# User Options
def options(*args, **kwds):
    parser = argparse.ArgumentParser(prog = 'CDhit2SQL',  formatter_class = argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-d', '--database',     dest     = "database", 
                        action = "store",       required = True,  help = "CDhit output file.")
    parser.add_argument('-s', '--sql',          dest     = "sqlfile", 
                        action = "store",       required = True,  help = "Output SQL file")
    parser.add_argument('-v', '--verbose',      dest     = "verbose", 
                        action = "store_true",  default  = False, help = "Verbose Mode.")
    options = parser.parse_args()
    return options

def CDhit2SQL(database, sqlfile, verbose):
   
    if verbose: sys.stderr.write("Retrieving data from {0} ...\n".format(database))
    cdhit = CDhit(database)

    if verbose: sys.stderr.write("Writing {0} ....\n".format(sqlfile))
    Path.mkdir(os.path.split(os.path.abspath(sqlfile))[0])
    sql_fd = gzip.open(sqlfile, 'wb')
    sql_fd.write(start_transaction())

    sql_fd.write(cdhit.toSQL())

    sql_fd.write(end_transaction())
    sql_fd.close()
    if verbose: sys.stderr.write("End execution.\n")

if __name__ == "__main__":
    options = options()
    CDhit2SQL(options.database, options.sqlfile, options.verbose)

    